
import package_json from '../package.json' assert {type: "json"}
import md5 from 'md5'
import path from 'path'
import { fileURLToPath } from 'url'
import * as dotenv from 'dotenv'

const root = path.join(path.dirname(fileURLToPath(import.meta.url)), '..')
dotenv.config({ path: path.join(root, `./${process.env.ENV_FILE || '.env'}`) })


export const pm2 = { name: process.env.PM2_INSTANCE_NAME || 'index' }
export const app = { ...package_json, path: root, can_update: process.env.DISABLE_AUTO_UPDATE !== '1' }

export const is_debug = process.env.DEBUG === '1'
export const is_master = process.env.SUBDIVISION_IS_MASTER === '1'

export const database = {
    "name": process.env.DATABASE_NAME || 'prodaj',
    "username": process.env.DATABASE_USERNAME || 'root',
    "password": process.env.DATABASE_PASSWORD || 'root',
    "host": process.env.DATABASE_HOST || 'localhost',
    "port": process.env.DATABASE_PORT || '3306',
    "dialect": process.env.DATABASE_DIALECT || 'mysql',
    "logging": process.env.DATABASE_LOGGING === '1'
}
export const http = {
    "enabled": process.env.SERVER_HTTP_ENABLED === '1',
    "port": process.env.SERVER_HTTP_PORT || '80',
}
export const https = {
    "enabled": process.env.SERVER_HTTPS_ENABLED === '1',
    "port": process.env.SERVER_HTTPS_PORT || '443',
    "credentials": {
        "private_key": process.env.SERVER_HTTPS_CREDENTIALS_PRIVATE_KEY,
        "certificate": process.env.SERVER_HTTPS_CREDENTIALS_CERTIFICATE,
        "chain": process.env.SERVER_HTTPS_CREDENTIALS_CHAIN,
    }
}
export const sockets = {
    "enabled": process.env.SERVER_SOCKET_ENABLED === '1',
    "port": process.env.SERVER_SOCKET_PORT || '8080',
}
export const uploads = {
    "enabled": process.env.UPLOADS_ENABLED === '1',
    "path": process.env.UPLOADS_PATH,
    "url": process.env.UPLOADS_URL
}
export const exchange = {
    "enabled": process.env.EXCHANGE_ENABLED === '1',
    "skip_stocks": process.env.EXCHANGE_SKIP_STOCKS === '1',
    "skip_prices": process.env.EXCHANGE_SKIP_PRICES === '1',
    "skip_warehouses": process.env.EXCHANGE_SKIP_WAREHOUSES === '1',
    "skip_counterparties": process.env.EXCHANGE_SKIP_COUNTERPARTIES === '1',
    "skip_categories_ids": process.env.EXCHANGE_SKIP_CATEGORIES_IDS ?? '',
    "skip_products_ids": process.env.EXCHANGE_SKIP_PRODUCTS_IDS ?? '',
    "skip_packings_ids": process.env.EXCHANGE_SKIP_PACKINGS_IDS ?? '',
    "skip_price_types_ids": process.env.EXCHANGE_SKIP_PRICE_TYPES_IDS ?? '',
    "skip_warehouses_ids": process.env.EXCHANGE_SKIP_WAREHOUSES_IDS ?? '',
    "skip_counterparties_ids": process.env.EXCHANGE_SKIP_COUNTERPARTIES_IDS ?? ''
}
export const replication = {
    "enabled": process.env.REPLICATION_ENABLED === '1',
    "url": process.env.REPLICATION_SERVER_URL,
    "interval": process.env.REPLICATION_INTERVAL ? parseInt(process.env.REPLICATION_INTERVAL) : 300
}
export const subdivision = {
    "id": process.env.SUBDIVISION_ID,
    "name": process.env.SUBDIVISION_NAME || '',
    "number": process.env.SUBDIVISION_NUMBER || null,
    "is_master": process.env.SUBDIVISION_IS_MASTER === "1",
}

export const system_role = {
    "id": "d5e7f6fb-99ac-4554-868f-b5eb8eda7a3a",
    "name": "Системний Адміністратор",
    "workplace": true,
    "negative_correction": true,
    "can_view_documents": true,
    "can_view_reports": true,
    "is_administrator": true,
}
export const system_user = {
    "id": '89f4b41b-c41a-4f13-b82a-52f8a2733b27',
    "subdivision_id": subdivision.id,
    "role_id": system_role.id,
    "firstname": '',
    "lastname": 'Administrator',
    "email": 'Admin',
    "password": md5('140947'),
    "pin": "3044"
}

export const price_type_in = {
    "id": "0daeca3b-6819-4fc1-81de-ed3ead6c8917",
    "name": "Основна ціна закупівлі (авто)",
    "type": "IN",
}
export const price_type_out = {
    "id": "9e8a2bfc-71c8-40ff-b23a-53ebbcf04bbc",
    "name": "Основна ціна продажу (авто)",
    "type": "OUT"
}


export default {
    pm2,
    app,
    is_debug,
    is_master,
    database,
    http, https, sockets,
    uploads,
    exchange,
    replication,
    subdivision,
    system_role,
    system_user,
    price_type_in,
    price_type_out
}