
import { v4 as uuid } from 'uuid'
import config from './../../config/index.js'
import instance from './instance.js'

import AcquiringTerminal from './schemes/dictionaries/devices/acquiring_terminal.js'
import FiscalRegister from './schemes/dictionaries/devices/fiscal_register.js'
import Scales from './schemes/dictionaries/devices/scales.js'

import PrescriptionSuppliers from './schemes/dictionaries/devices/prescription_suppliers.js'
import Workplace from './schemes/dictionaries/system/workplace.js'
import Subdivision from './schemes/dictionaries/system/subdivisions.js'

import TransactionsArticle from './schemes/dictionaries/system/transactions_article.js'
import PriceType from './schemes/dictionaries/system/price_type.js'
import Role from './schemes/dictionaries/system/role.js'
import Tax from './schemes/dictionaries/system/tax.js'
import Staff from './schemes/dictionaries/system/staff.js'
import Counterparty from './schemes/dictionaries/system/counterparty.js'
import CounterpartyProduct from './schemes/dictionaries/system/counterparty_product.js'

import Settings from './schemes/dictionaries/system/settings.js'
import Warehouse from './schemes/dictionaries/system/warehouse.js'



import ProductKind from './schemes/dictionaries/products/product_kind.js'
import ProductCategory from './schemes/dictionaries/products/product_category.js'
import ProductProducer from './schemes/dictionaries/products/product_producer.js'
import ProductActiveSubstance from './schemes/dictionaries/products/product_active_substance.js'
import ProductCharacteristic from './schemes/dictionaries/products/product_characteristic.js'

import ProductShortcut from './schemes/dictionaries/products/product_shortcut.js'
import ProductBarcode from './schemes/dictionaries/products/product_barcode.js'
import ProductPacking from './schemes/dictionaries/products/product_packing.js'
import ProductSeries from './schemes/dictionaries/products/product_series.js'

import Product from './schemes/dictionaries/products/product.js'
import Unit from './schemes/dictionaries/products/unit.js'


import ShiftDocument from './schemes/documents/shift/document.js'
import ShiftDocumentTransaction from './schemes/documents/shift/transaction.js'

import PaymentMethod from './schemes/dictionaries/system/payment_method.js'


import ArrivalDocument from './schemes/documents/arrival/document.js'
import ArrivalDocumentGood from './schemes/documents/arrival/good.js'


import ReturnDocument from './schemes/documents/return/document.js'
import ReturnDocumentGood from './schemes/documents/return/good.js'

import PricingDocument from './schemes/documents/pricing/document.js'
import PricingDocumentGood from './schemes/documents/pricing/good.js'

import InventoryDocument from './schemes/documents/inventory/document.js'
import InventoryDocumentGood from './schemes/documents/inventory/good.js'

import WriteoffDocument from './schemes/documents/writeoff/document.js'
import WriteoffDocumentGood from './schemes/documents/writeoff/good.js'

import DraftDocument from './schemes/documents/draft/document.js'
import DraftDocumentGood from './schemes/documents/draft/good.js'

import ReceiptDocument from './schemes/documents/receipt/document.js'
import ReceiptDocumentGood from './schemes/documents/receipt/good.js'
import ReceiptDocumentPayment from './schemes/documents/receipt/payment.js'

import ReportDocument from './schemes/documents/report/document.js'
import ReportDocumentGood from './schemes/documents/report/good.js'

import MoneyRegister from './schemes/registers/money/index.js'
import SalesRegister from './schemes/registers/sales/index.js'

import ProductPriceRegister from './schemes/registers/product/product_price.js'
import ProductPriceRegisterValues from './schemes/registers/product/product_price_values.js'

import ProductStockRegister from './schemes/registers/product/product_stock.js'
import ProductStockRegisterValues from './schemes/registers/product/product_stock_values.js'

import Errors from './schemes/system/errors.js'
import Notifications from './schemes/system/notifications.js'


import Migration from './migration/index.js'


Workplace.hasMany(ShiftDocument, { sourceKey: 'id', foreignKey: 'workplace_id' });
Workplace.hasMany(DraftDocument, { sourceKey: 'id', foreignKey: 'workplace_id' });
Workplace.belongsTo(FiscalRegister, { targetKey: 'id', foreignKey: 'register_id', as: 'register' });
Workplace.belongsTo(Scales, { targetKey: 'id', foreignKey: 'scale_id', as: 'scale' });

Warehouse.belongsTo(PriceType, { targetKey: 'id', foreignKey: 'price_type_id', as: 'price_type' });

Role.hasMany(Staff, { sourceKey: 'id', foreignKey: 'role_id' });

Staff.hasMany(ShiftDocument, { sourceKey: 'id', foreignKey: 'staff_id' });
Staff.hasMany(DraftDocument, { sourceKey: 'id', foreignKey: 'staff_id' });
Staff.belongsTo(Role, { targetKey: 'id', foreignKey: 'role_id' });

Tax.hasMany(Product, { sourceKey: 'id', foreignKey: 'tax_id' });

Product.hasOne(Tax, { sourceKey: 'tax_id', foreignKey: 'id', as: 'tax', constraints: false, allowNull: true });
Product.hasOne(ProductKind, { sourceKey: 'kind_id', foreignKey: 'id', as: 'kind', constraints: false, allowNull: true });
Product.hasOne(ProductCategory, { sourceKey: 'category_id', foreignKey: 'id', as: 'category', constraints: false, allowNull: true });
Product.hasOne(ProductProducer, { sourceKey: 'producer_id', foreignKey: 'id', as: 'producer', constraints: false, allowNull: true })
Product.hasOne(ProductActiveSubstance, { sourceKey: 'active_substance_id', foreignKey: 'id', as: 'active_substance', constraints: false, allowNull: true })
Product.hasMany(ProductBarcode, { sourceKey: 'id', foreignKey: 'product_id', as: 'barcodes' })

Product.hasMany(ProductPacking, { sourceKey: 'id', foreignKey: 'product_id', as: 'packings', constraints: false })
Product.hasMany(ProductSeries, { sourceKey: 'id', foreignKey: 'product_id', as: 'series' });


Counterparty.hasMany(CounterpartyProduct, { sourceKey: 'id', foreignKey: 'counterparty_id', as: 'items' })
CounterpartyProduct.belongsTo(Counterparty, { targetKey: 'id', foreignKey: 'counterparty_id', as: 'counterpary' })
CounterpartyProduct.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id', as: 'product' })
CounterpartyProduct.belongsTo(ProductPacking, { targetKey: 'id', foreignKey: 'packing_id', as: 'packing' })


Unit.hasMany(ProductPacking, { sourceKey: 'id', foreignKey: 'unit_id', as: 'packings' });


ProductSeries.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id' });

ProductPacking.belongsTo(Unit, { targetKey: 'id', foreignKey: 'unit_id', as: 'unit' });
ProductPacking.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id', as: 'product' });

ProductBarcode.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id', as: 'product' })
ProductBarcode.belongsTo(ProductPacking, { targetKey: 'id', foreignKey: 'packing_id', as: 'packing' })

ProductShortcut.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id', as: 'product' })
ProductShortcut.belongsTo(ProductPacking, { targetKey: 'id', foreignKey: 'packing_id', as: 'packing' })

PaymentMethod.belongsTo(AcquiringTerminal, { targetKey: 'id', foreignKey: 'acquiring_id', as: 'acquiring' });

ShiftDocument.hasMany(ShiftDocumentTransaction, { sourceKey: 'id', foreignKey: 'shift_id', as: 'transactions' });
ShiftDocument.hasOne(ShiftDocumentTransaction, { sourceKey: 'id', foreignKey: 'shift_id', as: 'open_transaction' });
ShiftDocument.hasOne(ShiftDocumentTransaction, { sourceKey: 'id', foreignKey: 'shift_id', as: 'close_transaction' });
ShiftDocument.belongsTo(Workplace, { targetKey: 'id', foreignKey: 'workplace_id' });
ShiftDocument.belongsTo(Staff, { targetKey: 'id', foreignKey: 'staff_id' });


ShiftDocumentTransaction.belongsTo(ShiftDocument, { targetKey: 'id', foreignKey: 'shift_id' });
ShiftDocumentTransaction.belongsTo(TransactionsArticle, { targetKey: 'id', foreignKey: 'article_id', as: 'article' })


DraftDocument.hasMany(DraftDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'goods' });
DraftDocument.belongsTo(Staff, { targetKey: 'id', foreignKey: 'staff_id' });
DraftDocument.belongsTo(Workplace, { targetKey: 'id', foreignKey: 'workplace_id' });

DraftDocumentGood.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id' });
DraftDocumentGood.belongsTo(ProductSeries, { targetKey: 'id', foreignKey: 'series_id' });
DraftDocumentGood.belongsTo(ProductPacking, { targetKey: 'id', foreignKey: 'packing_id', as: 'packing' });


ReceiptDocument.hasMany(ReceiptDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'goods' });
ReceiptDocument.hasMany(ReceiptDocumentPayment, { sourceKey: 'id', foreignKey: 'document_id', as: 'payments' });
ReceiptDocument.belongsTo(DraftDocument, { targetKey: 'id', foreignKey: 'basis_id' });
ReceiptDocument.belongsTo(Staff, { targetKey: 'id', foreignKey: 'staff_id' });
ReceiptDocument.belongsTo(Workplace, { targetKey: 'id', foreignKey: 'workplace_id' });
ReceiptDocument.belongsTo(FiscalRegister, { targetKey: 'id', foreignKey: 'register_id', as: 'register' });
ReceiptDocument.belongsTo(ShiftDocument, { targetKey: 'id', foreignKey: 'shift_id' });
ReceiptDocumentGood.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id' });
ReceiptDocumentGood.belongsTo(ProductSeries, { targetKey: 'id', foreignKey: 'series_id' });
ReceiptDocumentGood.belongsTo(ReceiptDocument, { targetKey: 'id', foreignKey: 'document_id', as: 'document' });
ReceiptDocumentPayment.belongsTo(PaymentMethod, { targetKey: 'id', foreignKey: 'method_id', as: 'method' });
ReceiptDocumentPayment.belongsTo(ReceiptDocument, { targetKey: 'id', foreignKey: 'document_id', as: 'document' });


ReportDocument.hasMany(ReportDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'goods' });
ReportDocument.belongsTo(Staff, { targetKey: 'id', foreignKey: 'staff_id' });
ReportDocument.belongsTo(Workplace, { targetKey: 'id', foreignKey: 'workplace_id' });
ReportDocument.belongsTo(ShiftDocument, { targetKey: 'id', foreignKey: 'shift_id' });
ReportDocument.belongsTo(Warehouse, { targetKey: 'id', foreignKey: 'warehouse_id' });

ReportDocumentGood.belongsTo(Product, { targetKey: 'id', foreignKey: 'product_id' });
ReportDocumentGood.belongsTo(ProductSeries, { targetKey: 'id', foreignKey: 'series_id' });


ArrivalDocument.belongsTo(Counterparty, { targetKey: 'id', foreignKey: 'counterparty_id', as: 'counterparty' })
ArrivalDocument.belongsTo(Warehouse, { targetKey: 'id', foreignKey: 'warehouse_id', as: 'warehouse' })
ArrivalDocument.belongsTo(PriceType, { targetKey: 'id', foreignKey: 'price_type_id', as: 'price_type' })
ArrivalDocument.hasMany(ArrivalDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'items' })

ArrivalDocumentGood.belongsTo(Product, { foreignKey: 'product_id', targetKey: 'id', as: 'product' })
ArrivalDocumentGood.belongsTo(ProductSeries, { foreignKey: 'series_id', targetKey: 'id', as: 'series' })
ArrivalDocumentGood.belongsTo(ProductPacking, { foreignKey: 'packing_id', targetKey: 'id', as: 'packing' })

ReturnDocument.belongsTo(Counterparty, { targetKey: 'id', foreignKey: 'counterparty_id', as: 'counterparty' })
ReturnDocument.belongsTo(Warehouse, { targetKey: 'id', foreignKey: 'warehouse_id', as: 'warehouse' })
ReturnDocument.belongsTo(PriceType, { targetKey: 'id', foreignKey: 'price_type_id', as: 'price_type' })
ReturnDocument.hasMany(ReturnDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'items' })

ReturnDocumentGood.belongsTo(Product, { foreignKey: 'product_id', targetKey: 'id', as: 'product' })
ReturnDocumentGood.belongsTo(ProductSeries, { foreignKey: 'series_id', targetKey: 'id', as: 'series' })
ReturnDocumentGood.belongsTo(ProductPacking, { foreignKey: 'packing_id', targetKey: 'id', as: 'packing' })


PricingDocument.belongsTo(ArrivalDocument, { targetKey: 'id', foreignKey: 'basis_id', as: 'basis' })
PricingDocument.belongsTo(PriceType, { targetKey: 'id', foreignKey: 'price_type_id', as: 'price_type' })
PricingDocument.hasMany(PricingDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'items' })

PricingDocumentGood.belongsTo(Product, { foreignKey: 'product_id', targetKey: 'id', as: 'product' })
PricingDocumentGood.belongsTo(ProductSeries, { foreignKey: 'series_id', targetKey: 'id', as: 'series' })
PricingDocumentGood.belongsTo(ProductPacking, { foreignKey: 'packing_id', targetKey: 'id', as: 'packing' })



InventoryDocument.belongsTo(Warehouse, { targetKey: 'id', foreignKey: 'warehouse_id', as: 'warehouse' })
InventoryDocument.hasMany(InventoryDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'items' })
InventoryDocumentGood.belongsTo(Product, { foreignKey: 'product_id', targetKey: 'id', as: 'product' })
InventoryDocumentGood.belongsTo(ProductSeries, { foreignKey: 'series_id', targetKey: 'id', as: 'series' })
InventoryDocumentGood.belongsTo(ProductPacking, { foreignKey: 'packing_id', targetKey: 'id', as: 'packing' })

WriteoffDocument.belongsTo(Warehouse, { targetKey: 'id', foreignKey: 'warehouse_id', as: 'warehouse' })
WriteoffDocument.hasMany(WriteoffDocumentGood, { sourceKey: 'id', foreignKey: 'document_id', as: 'items' })
WriteoffDocumentGood.belongsTo(Product, { foreignKey: 'product_id', targetKey: 'id', as: 'product' })
WriteoffDocumentGood.belongsTo(ProductSeries, { foreignKey: 'series_id', targetKey: 'id', as: 'series' })
WriteoffDocumentGood.belongsTo(ProductPacking, { foreignKey: 'packing_id', targetKey: 'id', as: 'packing' })

class Database {

    static ready = false


    static prepare(payload, include = [], exclude = [], additional = {}) {
        const body = {}
        const keys = include && Array.isArray(include) && include.length ? include : Object.keys(payload)
        keys.map(key => { if (payload[key] !== undefined) body[key] = payload[key] })
        if (exclude) exclude.map(key => delete body[key])
        return Object.assign({}, body, additional)
    }


    static chunk = (array, size) => {
        if (!array) return []
        const result = []
        for (let i = 0; i < array.length; i += size) {
            result.push(array.slice(i, i + size))
        }
        return result
    }

    static instance = instance

    static Settings = Settings

    static Subdivisions = Subdivision

    static Roles = Role
    static Staff = Staff
    static Counterparties = Counterparty
    static TransactionsArticles = TransactionsArticle


    static PriceTypes = PriceType
    static ProductCategories = ProductCategory
    static ProductKinds = ProductKind
    static ProductProducers = ProductProducer
    static ProductActiveSubstances = ProductActiveSubstance

    static ProductPacking = ProductPacking
    static ProductCharacteristic = ProductCharacteristic
    static ProductShortcats = ProductShortcut

    static Products = Product
    static Series = ProductSeries
    static Barcodes = ProductBarcode
    static Units = Unit
    static Taxes = Tax

    static Warehouses = Warehouse
    static Workplaces = Workplace
    static PaymentMethods = PaymentMethod
    static AcquiringTerminals = AcquiringTerminal
    static Scales = Scales
    static FiscalRegisters = FiscalRegister
    static PrescriptionSuppliers = PrescriptionSuppliers

    static CounterpartiesProducts = CounterpartyProduct

    static Documents = class Documents {
        static Shifts = ShiftDocument
        static ShiftsTransactions = ShiftDocumentTransaction

        static Arrival = ArrivalDocument
        static ArrivalGoods = ArrivalDocumentGood

        static Return = ReturnDocument
        static ReturnGoods = ReturnDocumentGood

        static Pricing = PricingDocument
        static PricingGoods = PricingDocumentGood

        static Inventory = InventoryDocument
        static InventoryGoods = InventoryDocumentGood

        static Writeoff = WriteoffDocument
        static WriteoffGoods = WriteoffDocumentGood

        static Draft = DraftDocument
        static DraftGoods = DraftDocumentGood

        static Receipt = ReceiptDocument
        static ReceiptGoods = ReceiptDocumentGood
        static ReceiptPayment = ReceiptDocumentPayment

        static Report = ReportDocument
        static ReportGoods = ReportDocumentGood
    }

    static Registers = class Registers {
        static Money = MoneyRegister
        static Sales = SalesRegister

        static ProductStock = ProductStockRegister
        static ProductStockValues = ProductStockRegisterValues

        static ProductPrice = ProductPriceRegister
        static ProductPriceValues = ProductPriceRegisterValues
    }

    static System = class System {
        static Errors = Errors
        static Notifications = Notifications
    }

    static Error = class RequestError extends Error {
        constructor(message, status = 400) {
            super(message)
            this.status = status
        }
    }

    static async onCatchError(error, { name, type = "", subdivision_id, }) {
        try {
            const row = await Database.System.Errors.create({ id: uuid(), subdivision_id, name, type, message: error.message, stack: error.stack + (error?.original?.text ? `\n\n${error?.original?.text}` : '') })
            return row.getDataValue('id')
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, "Помилка запису логу: ", JSON.stringify({ id: uuid(), subdivision_id, name, type, message: error.message, stack: error.stack }))
        }
    }
}


instance.afterBulkSync(onDatabaseReady)
instance.sync({ hooks: true })



async function onDatabaseReady() {
    await createViews()
    await Migration.start()
}

async function createViews() {
    try {
        switch (config.database.dialect) {
            case 'mysql':
            case 'mariadb':
                await Database.instance.query(`CREATE OR REPLACE VIEW products_stock_register_view AS SELECT products_stock_register_values.product_id AS product_id, products_stock_register_values.series_id AS series_id, products_stock_register_values.warehouse_id AS warehouse_id, products_stock_register_values.assignment_id AS assignment_id, sum(products_stock_register_values.in_stock) AS in_stock, sum(products_stock_register_values.to_ship) AS to_ship FROM products_stock_register_values GROUP BY products_stock_register_values.product_id, products_stock_register_values.series_id, products_stock_register_values.warehouse_id, products_stock_register_values.assignment_id`)
                break
            default:
                console.error('DATABASE ERROR: Неможливо створити представлення. Нередбачений діалект бази даних.')
        }
    } catch (error) {
        console.log(`[${new Date().toISOString()}] `, 'Помилка створення представлень бази даних', error)
    }
}


export default Database
