
import { col, where, and, or, literal, DataTypes, Op, HasOne, BelongsTo } from "@sequelize/core"
import semver from 'semver'
import config from "../../../config/index.js"
import instance from "../instance.js"
import { Column, Index, Constraint } from './utils.js'
import Database from "../index.js"
import SettingsHelper from "../schemes/dictionaries/system/settings_helper.js"
import { TRANSACTION_ARTICLES } from "../../../constants/index.js"

export const DB_VERSION_KEY = 'database_version'

const MIGRATIONS = {}

MIGRATIONS['1.1.34+7'] = async (db, transaction) => {
    await Column.add(db, transaction, 'receipt_documents_payments', 'amount_rounded', { type: DataTypes.REAL, allowNull: true })
}

MIGRATIONS['1.1.34+3'] = async (db, transaction) => {
    await Column.add(db, transaction, 'payments_methods', 'rounding', { type: DataTypes.INTEGER, allowNull: false, defaultValue: 1 })
    await Column.add(db, transaction, 'payments_methods', 'rounding_rule', { type: DataTypes.STRING(10), allowNull: false, defaultValue: 'MATH' })
}

MIGRATIONS['1.1.34+2'] = async (db, transaction) => {
    await Column.add(db, transaction, 'fiscal_registers', 'host', { type: DataTypes.STRING(36), allowNull: true })
    await Column.add(db, transaction, 'fiscal_registers', 'port', { type: DataTypes.STRING(10), allowNull: true })
}

MIGRATIONS['1.1.34'] = async (db, transaction) => {
    await Column.change(db, transaction, 'fiscal_registers', 'license', { type: DataTypes.STRING, allowNull: true })
    await Column.change(db, transaction, 'fiscal_registers', 'login', { type: DataTypes.STRING, allowNull: true })
    await Column.change(db, transaction, 'fiscal_registers', 'password', { type: DataTypes.STRING, allowNull: true })
    await Column.change(db, transaction, 'fiscal_registers', 'number', { type: DataTypes.STRING(24), allowNull: true })
}

MIGRATIONS['1.1.33+2'] = async (db, transaction) => {
    await Column.add(db, transaction, 'pricing_documents', 'name', { type: DataTypes.STRING, allowNull: true, defaultValue: null })
}

MIGRATIONS['1.1.33'] = async (db, transaction) => {
    await Column.add(db, transaction, 'roles', 'can_view_statistic', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_reports_sale', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_reports_stock', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_reports_money', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_documents', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_shared_dictonaries', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_products', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_categories', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_taxes', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_price_types', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_roles', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_counterparties', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_transactions_articles', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_individual_dictonaries', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_shortcuts', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_warehouses', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_staff', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_workplaces', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_payment_methods', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_registers', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_acquirings', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_scales', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_settings', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })

    const roles = await Database.Roles.findAll({ transaction })
    await Promise.all(roles.map(role => role.update({ can_view_documents: role.can_manage_warehouse_accounting })))
}

MIGRATIONS['1.1.32+5'] = async (db, transaction) => {
    await Constraint.remove(db, transaction, 'writeoff_documents_goods', 'writeoff_documents_goods_ibfk_1')
    await Constraint.addForeign(db, transaction, 'writeoff_documents_goods', { fields: ['document_id'], name: 'writeoff_documents_goods_ibfk_1', references: { table: 'writeoff_documents', field: 'id' }, })
}

MIGRATIONS['1.1.32+3'] = async (db, transaction) => {
    await Column.add(db, transaction, 'workplaces', 'printing', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: true })
}

MIGRATIONS['1.1.32+2'] = async (db, transaction) => {
    await Column.change(db, transaction, 'counterparties', 'name', { type: DataTypes.STRING, allowNull: false, defaultValue: '' })
    await Column.change(db, transaction, 'counterparties', 'email', { type: DataTypes.STRING, allowNull: false, defaultValue: '' })
}

MIGRATIONS['1.1.30+2'] = async (db, transaction) => {
    await Column.add(db, transaction, 'price_types', 'rounding', { type: DataTypes.INTEGER, allowNull: false, defaultValue: 1 })
    await Column.add(db, transaction, 'price_types', 'rounding_rule', { type: DataTypes.STRING(10), allowNull: false, defaultValue: "MATH" })
}


MIGRATIONS['1.1.30'] = async (db, transaction) => {
    await Column.add(db, transaction, 'roles', 'can_view_arrivals', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_returns', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_pricing', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_inventory', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_writeoff', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Column.add(db, transaction, 'roles', 'can_view_only_real_values_in_inventory', { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false })
    await Index.add(db, transaction, 'products_barcodes', { name: 'barcodes_fts', fields: ['barcode'], type: 'FULLTEXT' })
    await Index.add(db, transaction, 'products_packings', { name: 'packings_fts', fields: ['name'], type: 'FULLTEXT' })
    await Index.add(db, transaction, 'products_categories', { name: 'categories_fts', fields: ['name'], type: 'FULLTEXT' })
}
MIGRATIONS['1.1.27'] = async (db, transaction) => {
    await Column.add(db, transaction, 'money_register', 'operation_article_id', { type: DataTypes.STRING(36), allowNull: true, references: { model: 'transactions_articles', key: 'id' } })
    await Column.add(db, transaction, 'return_documents', 'counterparty_id', { type: DataTypes.STRING(36), allowNull: true, defaultValue: null, references: { model: 'counterparties', key: 'id' } })
    await Column.add(db, transaction, 'arrival_documents', 'counterparty_id', { type: DataTypes.STRING(36), allowNull: true, defaultValue: null, references: { model: 'counterparties', key: 'id' } })
    await Column.add(db, transaction, 'transactions', 'article_id', { type: DataTypes.STRING(36), allowNull: true, references: { model: 'transactions_articles', key: 'id', } })
    await Index.add(db, transaction, 'transactions', { name: 'by_shift_id', fields: ['shift_id'], })
}
MIGRATIONS['1.1.22'] = async (db, transaction) => {
    await Column.add(db, transaction, 'price_types', 'type', { type: DataTypes.STRING(24), allowNull: false })
    await Database.PriceTypes.update({ type: 'IN' }, { where: { name: { [Op.like]: '%закупівлі%' } }, transaction })
    await Database.PriceTypes.update({ type: 'OUT' }, { where: { name: { [Op.like]: '%продажу%' } }, transaction })
}
MIGRATIONS['1.1.20'] = async (db, transaction) => {
    await Column.add(db, transaction, 'fiscal_registers', 'number', { type: DataTypes.STRING(24), allowNull: false })
}
MIGRATIONS['1.1.19'] = async (db, transaction) => {
    await Column.add(db, transaction, 'products', 'name_in_receipt', { type: DataTypes.STRING(64), allowNull: false, defaultValue: '' })
}

export default class Migration {

    static async start() {
        const database = instance.getQueryInterface()
        const transaction = await database.sequelize.transaction()
        try {
            const current = await SettingsHelper.get(DB_VERSION_KEY)
            if (!current) {
                await SettingsHelper.set({ [DB_VERSION_KEY]: config.app.version }, { transaction })
            }
            if (!current || current === config.app.version) {
                console.log(`[${new Date().toISOString()}] `, `База даних актуальної версії: ${current || config.app.version}`)
                await transaction.commit()
                return Migration.onDatabaseReady()
            }
            console.log(`[${new Date().toISOString()}] `, `База даних потребує міграції: "${current}" -> "${config.app.version}"`)
            const versions = semver.sort(Object.keys(MIGRATIONS)).filter(version => (
                (
                    semver.gt(version, current) && // міграція на версії більші ніж поточна версія бази даних
                    semver.lte(version, config.app.version) // міграцію на версії менші за версію коду включно з нею
                ) ||
                semver.eq(version, current) && semver.compareBuild(version, current) === 1 // застосувати міграції поточної версії та більшого номеру білду
            ))
            for (let i = 0; i < versions.length; i++) {
                await MIGRATIONS[versions[i]](database, transaction)
            }
            await SettingsHelper.set({ [DB_VERSION_KEY]: config.app.version }, { transaction })
            await transaction.commit()
            console.log(`[${new Date().toISOString()}] `, 'Міграція виконана успішно')
            return Migration.onDatabaseReady()
        } catch (error) {
            await transaction.rollback()
            console.log(`[${new Date().toISOString()}] `, 'Міграція завершена з помилкою', error)
            throw error
        }

    }

    static async onDatabaseReady() {
        if (!await Database.Subdivisions.count({ where: { id: config.subdivision.id } })) await Database.Subdivisions.create(config.subdivision)
        if (!await Database.Roles.count({ where: { id: config.system_role.id } })) await Database.Roles.create(config.system_role)
        if (!await Database.Staff.count({ where: { id: config.system_user.id, subdivision_id: config.subdivision.id } })) await Database.Staff.create(config.system_user)
        if (!await Database.PriceTypes.count({ where: { id: config.price_type_in.id } })) await Database.PriceTypes.create(config.price_type_in)
        if (!await Database.PriceTypes.count({ where: { id: config.price_type_out.id } })) await Database.PriceTypes.create(config.price_type_out)

        const existed = await Database.TransactionsArticles.findAll({ where: { id: { [Op.in]: TRANSACTION_ARTICLES.map(it => it.id) } } })
        await Promise.all(TRANSACTION_ARTICLES.filter(article => !existed.map(it => it.id).includes(article.id)).map(article => Database.TransactionsArticles.create(article)))
        Database.ready = true
    }

}