import config from '../../../../../config/index.js'
import Database from '../../../index.js'

export default class SettingsHelper {

    static async get(key, options = {}) {
        const subdivision_id = options.subdivision_id ? options.subdivision_id : config.subdivision.id
        delete options.subdivision_id
        const record = await Database.Settings.findOne({ where: { subdivision_id, key }, ...options })
        switch (record?.type) {
            case undefined:
                return null
            case 'boolean':
                return record.value === 'true' || record.value === '1';
            case 'number':
                return parseFloat(record.value);
            default:
                return record.value;
        }
    }


    static async set(values, options = {}) {
        const transaction = options.transaction ? options.transaction : await Database.instance.transaction()
        try {
            const subdivision_id = options.subdivision_id ? options.subdivision_id : config.subdivision.id
            delete options.subdivision_id
            const keys = Object.keys(values).filter(key => ['boolean', 'number', 'string'].includes(typeof values[key]))
            const prepared = keys.map(key => ({ subdivision_id, key, type: typeof values[key], value: values[key].toString() }))
            await Promise.all(prepared.map(row => Database.Settings.upsert(row, { ...options, transaction })))
            if (!options.transaction) await transaction.commit()
        } catch (error) {
            if (!options.transaction) await transaction.rollback()
            throw error
        }
    }
}