import sequelize, { Model, DataTypes } from '@sequelize/core'
import { subdivision } from '../../../../../config/index.js'
import { TRANSACTION_PRESENTATIONS } from '../../../../../constants/index.js'
import instance from '../../../instance.js'


class Transaction extends Model { }
Transaction.init({
    id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true },
    subdivision_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true },
    shift_id: { type: DataTypes.STRING(36), allowNull: false, references: { model: 'shifts', key: 'id' } },
    article_id: { type: DataTypes.STRING(36), allowNull: true, references: { model: 'transactions_articles', key: 'id' } },
    presentation: { type: DataTypes.VIRTUAL, get: function () { return TRANSACTION_PRESENTATIONS[this.getDataValue('type')] || '' } },
    type: { type: DataTypes.STRING, allowNull: false }, // SHIFT_OPEN | SHIFT_CLOSE | CASH_IN | CASH_OUT | AUTO_CASH_IN | AUTO_CASH_OUT | GO_OFFLINE | GO_ONLINE
    amount: { type: DataTypes.REAL, allowNull: true, defaultValue: null },
    comment: { type: DataTypes.STRING, allowNull: true, defaultValue: null },
    fiscal_id: { type: DataTypes.STRING(36), allowNull: true, defaultValue: null },
    fiscal_code: { type: DataTypes.STRING(36), allowNull: true, defaultValue: null },
    fiscal_date: { type: DataTypes.DATE, allowNull: true, defaultValue: null },
}, {
    sequelize: instance,
    modelName: 'transactions',
    tableName: 'transactions',
    indexes: [
        { name: 'by_shift_id', fields: ['shift_id'], },
    ]
});

export default Transaction