import sequelize, { Model, DataTypes, } from '@sequelize/core'
import instance from '../../../instance.js'


class MoneyRegister extends Model { }
MoneyRegister.init({
    basis_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true }, // Transaction or Receipt payment
    subdivision_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, references: { model: 'subdivisions', key: 'id' } },
    workplace_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, references: { model: 'workplaces', key: 'id' } },
    operation: { type: DataTypes.STRING(20), allowNull: false, }, // CASH_IN CASH_OUT CASH_IN_AUTO CASH_OUT_AUTO SELL RETURN  
    operation_article_id: { type: DataTypes.STRING(36), allowNull: true, references: { model: 'transactions_articles', key: 'id' } },
    type: { type: DataTypes.STRING(10), allowNull: false, }, // CASH or CARD
    amount: { type: DataTypes.REAL, allowNull: false },
    period: { type: DataTypes.DATE, allowNull: false, defaultValue: DataTypes.NOW },
    deletion_mark: { type: DataTypes.BOOLEAN, allowNull: false, defaultValue: false }
}, {
    sequelize: instance,
    modelName: 'money_register',
    tableName: 'money_register',
    indexes: [
        { name: 'money_register_index', fields: ['basis_id', 'subdivision_id', 'workplace_id', 'type'], unique: false },
    ],
});

export default MoneyRegister

export class MoneyRegisterHelper {
    static async insert(payload, options = {}) {
        MoneyRegister.upsert(payload, options)
    }
}