import { Model, DataTypes } from '@sequelize/core'
import instance from '../../../instance.js'

class ProductPriceRegisterValues extends Model { }
ProductPriceRegisterValues.init({
    product_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, references: { model: 'products', key: 'id' } },
    series_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, }, //references: { model: 'products_series', key: 'id' } },
    packing_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, }, // references: { model: 'products_packings', key: 'id' } },
    price_type_id: { type: DataTypes.STRING(36), allowNull: true, primaryKey: true, }, // references: { model: 'price_types', key: 'id' } },
    price: { type: DataTypes.REAL, allowNull: false },
    period: { type: DataTypes.DATE, allowNull: false },
}, {
    sequelize: instance,
    modelName: 'products_price_register_values',
    tableName: 'products_price_register_values',
    indexes: [
        { name: 'products_price_register_values_index', fields: ['product_id', 'series_id', 'packing_id', 'price_type_id'], unique: false },
    ],
    timestamps: false
});
ProductPriceRegisterValues.removeAttribute('id');

export default ProductPriceRegisterValues


/*
DELIMITER $$
DROP PROCEDURE IF EXISTS products_price_register_update $$
CREATE PROCEDURE products_price_register_update (
    triger_product_id VARCHAR(36), 
    triger_series_id VARCHAR(36),
    triger_packing_id VARCHAR(36),
    triger_price_type_id VARCHAR(36)
)
BEGIN
        INSERT INTO products_price_register_values
            SELECT  
                product_id,
                series_id,
                packing_id,
                price_type_id,
                price,
                MAX(period) AS `period`,
                NOW() AS `createdAt`,
                NOW() AS `updatedAt`
            FROM  products_price_register
            WHERE 
                product_id = triger_product_id AND 
                series_id = triger_series_id AND 
                packing_id = triger_packing_id AND 
                price_type_id = triger_price_type_id
            GROUP BY product_id, series_id, packing_id, price_type_id
        ON DUPLICATE KEY UPDATE 
            price = VALUES(price),
            period = VALUES(period),
            updatedAt = VALUES(updatedAt);
END$$
DELIMITER ;
  
DROP TRIGGER IF EXISTS products_price_register_insert_triger;
CREATE TRIGGER products_price_register_insert_triger AFTER INSERT ON products_price_register FOR EACH ROW
  CALL products_price_register_update(NEW.product_id, NEW.series_id, NEW.packing_id, NEW.price_type_id);
  
DROP TRIGGER IF EXISTS products_price_register_update_triger;
CREATE TRIGGER products_price_register_update_triger AFTER UPDATE ON products_price_register FOR EACH ROW
  CALL products_price_register_update(NEW.product_id, NEW.series_id, NEW.packing_id, NEW.price_type_id);

*/