import { fn, col } from '@sequelize/core'
import Database from '../../../index.js'

export default class ProductPriceRegisterValuesHelper {

    static async get(options) {
        const transaction = options && options.transaction ? options.transaction : await Database.instance.transaction()
        try {
            const row = await Database.Registers.ProductPriceValues.findOne({ where: options && options.where ? options.where : {}, transaction })
            if (!options?.transaction) await transaction.commit()
            return row ? row.getDataValue('price') : null
        } catch (error) {
            if (!options?.transaction) await transaction.rollback()
            throw error
        }
    }

    static async getAll(options) {
        const transaction = options && options.transaction ? options.transaction : await Database.instance.transaction()
        try {
            const rows = await Database.Registers.ProductPriceValues.findAll({ where: options && options.where ? options.where : {}, transaction })
            if (!options?.transaction) await transaction.commit()
            return rows
        } catch (error) {
            if (!options?.transaction) await transaction.rollback()
            throw error
        }
    }



    static async recalculate(options) {
        const transaction = options && options.transaction ? options.transaction : await Database.instance.transaction()
        try {
            const where = options && options.where ? options.where : {}
            const rows = await Database.Registers.ProductPrice.findAll({
                attributes: {
                    include: [[fn('max', col('period')), 'period']],
                    exclude: ['basis_id', 'price', 'period', 'deletion_mark', 'createdAt', 'updatedAt']
                },
                where: { ...where, deletion_mark: false },
                group: ['product_id', 'series_id', 'packing_id', 'price_type_id'],
                transaction,
                raw: true
            })
            const values = await Promise.all(rows.map(row => Database.Registers.ProductPrice.findOne({
                attributes: {
                    exclude: ['basis_id', 'createdAt', 'updatedAt']
                },
                where: row,
                transaction,
                raw: true
            })))
            await Database.Registers.ProductPriceValues.destroy({ where, transaction })
            await Promise.all(values.map(item => Database.Registers.ProductPriceValues.upsert(item, { transaction })))
            if (!options?.transaction) await transaction.commit()
        } catch (error) {
            if (!options?.transaction) await transaction.rollback()
            throw error
        }
    }
}