import { Model, DataTypes } from '@sequelize/core'
import instance from '../../../instance.js'


class ProductStockRegister extends Model { }
ProductStockRegister.init({
    product_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, references: { model: 'products', key: 'id' } },
    series_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, }, // references: { model: 'products_series', key: 'id' } },
    warehouse_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, }, // references: { model: 'warehouses', key: 'id' } },
    assignment_id: { type: DataTypes.STRING(36), allowNull: false, primaryKey: true, },
    in_stock: { type: DataTypes.REAL, allowNull: false },
    to_ship: { type: DataTypes.REAL, allowNull: false },
    period: { type: DataTypes.DATE, allowNull: false, primaryKey: true }
}, {
    sequelize: instance,
    modelName: 'products_stock_register_values',
    tableName: 'products_stock_register_values',
    indexes: [
        { name: 'agregate_index', fields: ['product_id', 'series_id', 'warehouse_id', 'assignment_id'], unique: false, },
        { name: 'period_index', fields: ['period'], unique: false, },
    ],
    timestamps: false
});

export default ProductStockRegister

/*
DELIMITER $$
DROP PROCEDURE IF EXISTS products_stock_register_update $$
CREATE PROCEDURE products_stock_register_update (
    triger_product_id VARCHAR(36), 
    triger_series_id VARCHAR(36),
    triger_warehouse_id VARCHAR(36),
    triger_assignment_id VARCHAR(36)
)
BEGIN
       INSERT INTO products_stock_register_values
            SELECT  
                product_id,
                series_id,
                warehouse_id,
                assignment_id,
                ROUND(SUM(in_stock),3) AS `in_stock`,
                ROUND(SUM(to_ship),3) AS `to_ship`,
                MAX(period) AS `period`,
                CURRENT_TIMESTAMP() AS `createdAt`,
                CURRENT_TIMESTAMP() AS `updatedAt`
            FROM  products_stock_register
            WHERE
                product_id = triger_product_id AND
                series_id = triger_series_id AND
                warehouse_id = triger_warehouse_id AND
                assignment_id = triger_assignment_id
            GROUP BY product_id, series_id, warehouse_id, assignment_id
            HAVING in_stock > 0 
        ON DUPLICATE KEY UPDATE 
            in_stock = VALUES(in_stock),
            to_ship = VALUES(to_ship),
            updatedAt = VALUES(updatedAt);
        

        DELETE register_values FROM products_stock_register_values register_values
        INNER JOIN (
            SELECT  
                product_id,
                series_id,
                warehouse_id,
                assignment_id,
                ROUND(SUM(in_stock),3) AS `in_stock`,
                ROUND(SUM(to_ship),3) AS `to_ship`,
                MAX(period) AS `period`,
                CURRENT_TIMESTAMP() AS `createdAt`,
                CURRENT_TIMESTAMP() AS `updatedAt`
            FROM    products_stock_register
            WHERE
                product_id = triger_product_id AND
                series_id = triger_series_id AND
                warehouse_id = triger_warehouse_id AND
                assignment_id = triger_assignment_id
            GROUP BY product_id, series_id, warehouse_id, assignment_id
            HAVING in_stock <= 0 
        ) register ON (
            register.product_id = register_values.product_id AND 
            register.series_id = register_values.series_id AND 
            register.warehouse_id = register_values.warehouse_id AND 
            register.assignment_id = register_values.assignment_id 
        ); 
END$$
DELIMITER ;

DROP TRIGGER IF EXISTS products_stock_register_insert_triger;
CREATE TRIGGER products_stock_register_insert_triger AFTER INSERT ON products_stock_register FOR EACH ROW
  CALL products_stock_register_update(NEW.product_id, NEW.series_id, NEW.warehouse_id, NEW.assignment_id);

DROP TRIGGER IF EXISTS products_stock_register_update_triger;
CREATE TRIGGER products_stock_register_update_triger AFTER UPDATE ON products_stock_register FOR EACH ROW
  CALL products_stock_register_update(NEW.product_id, NEW.series_id, NEW.warehouse_id, NEW.assignment_id);



*/