import { fn, col, Op } from '@sequelize/core'
import moment from 'moment'
import Database from '../../../index.js'
import instance from '../../../instance.js'

class ProductStockRegisterValuesHelper {


    static async get({ product_id, series_id = '', warehouse_id, assignment_id = '' }) {
        return Database.Registers.ProductStock.sum('in_stock', {
            where: {
                product_id,
                series_id,
                warehouse_id,
                assignment_id,
                deletion_mark: false
            },
            group: ['product_id', 'series_id', 'warehouse_id', 'assignment_id']
        })
    }

    static async getAll({ where = {} }) {
        return Database.Registers.ProductStock.findAll({
            attributes: [
                'product_id',
                'series_id',
                'warehouse_id',
                'assignment_id',
                [fn('round', fn('sum', col('in_stock')), 3), 'in_stock'],
                [fn('round', fn('sum', col('to_ship')), 3), 'to_ship'],
            ],
            where: {
                ...where,
                deletion_mark: false
            },
            group: ['product_id', 'series_id', 'warehouse_id', 'assignment_id']
        })
    }


    // static async recalculate(options) {
    //     const transaction = options.transaction ? options.transaction : await Database.instance.transaction()
    //     try {
    //         const current_month = moment().startOf('month').toISOString()
    //         const current_year = moment().startOf('year').toISOString()

    //         const where_condition = options && options.where ? { ...options.where } : {}

    //         const prev_years_rows = await Database.Registers.ProductStock.findAll({
    //             attributes: {
    //                 include: [
    //                     [fn('sum', col('in_stock')), 'in_stock'],
    //                     [fn('sum', col('to_ship')), 'to_ship'],
    //                     [fn('concat', fn("date_format", col("period"), "%Y"), '-01-01T00:00:00.000Z'), "period"]
    //                 ],
    //                 exclude: ['basis_id', 'in_stock', 'to_ship', 'period', 'deletion_mark', 'createdAt', 'updatedAt']
    //             },
    //             where: {
    //                 ...where_condition,
    //                 period: { [Op.lt]: current_year },
    //                 deletion_mark: false
    //             },
    //             group: ['product_id', 'series_id', 'warehouse_id', 'assignment_id', 'period'],
    //             transaction
    //         })


    //         await Database.Registers.ProductStockValues.destroy({ where: { ...where_condition, period: { [Op.lt]: current_year } }, transaction })

    //         await Promise.all(prev_years_rows.map(row => row.get({ plain: true })).map(item => Database.Registers.ProductStockValues.upsert(item, { transaction })))

    //         const prev_months_rows = await Database.Registers.ProductStock.findAll({
    //             attributes: {
    //                 include: [
    //                     [fn('sum', col('in_stock')), 'in_stock'],
    //                     [fn('sum', col('to_ship')), 'to_ship'],
    //                     [fn('concat', fn("date_format", col("period"), "%Y-%m"), '-01T00:00:00.000Z'), "period"],
    //                 ],
    //                 exclude: ['basis_id', 'in_stock', 'to_ship', 'period', 'deletion_mark', 'createdAt', 'updatedAt']
    //             },
    //             where: {
    //                 [Op.and]: [
    //                     { period: { [Op.lt]: current_month } },
    //                     { period: { [Op.gte]: current_year } },
    //                     ...Object.keys(options && options.where ? options.where : {}).map((key) => ({ [key]: options.where[key] })),
    //                     { deletion_mark: false }
    //                 ]
    //             },
    //             group: ['product_id', 'series_id', 'warehouse_id', 'assignment_id', 'period'],
    //             transaction
    //         })
    //         await Database.Registers.ProductStockValues.destroy({
    //             where: {
    //                 [Op.and]: [
    //                     { period: { [Op.lt]: current_month } },
    //                     { period: { [Op.gte]: current_year } },
    //                     ...Object.keys(where_condition).map((key) => ({ [key]: options.where[key] }))
    //                 ]
    //             },
    //             transaction
    //         })

    //         await Promise.all(prev_months_rows.map(row => row.get({ plain: true })).map(item => Database.Registers.ProductStockValues.upsert(item, { transaction })))

    //         const current_months_rows = await Database.Registers.ProductStock.findAll({
    //             attributes: {
    //                 exclude: ['basis_id', 'createdAt', 'updatedAt']
    //             },
    //             where: {
    //                 period: { [Op.gte]: current_month },
    //                 ...(options && options.where ? options.where : {})
    //             },
    //             transaction
    //         })
    //         await Database.Registers.ProductStockValues.destroy({ where: { ...where_condition, period: { [Op.gte]: current_month } }, transaction })
    //         await Promise.all(current_months_rows.map(row => row.get({ plain: true })).map(item => Database.Registers.ProductStockValues.upsert(item, { transaction })))

    //         if (!options?.transaction) await transaction.commit()
    //     } catch (error) {
    //         if (!options?.transaction) await transaction.rollback()
    //         throw error
    //     }
    // }





}

export default ProductStockRegisterValuesHelper