import Round from '../lib/Round.js'

export default {
    // >>> Дані пакету <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    "status": "Status",
    "message": "Message",
    "message_number": "MessageNo",
    "receive_from": "ReceiveFrom",

    // >>> Довідники <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    "products": {
        "key": "Goods",
        "structure": {
            "id": "ProductID",
            "name": "Name",
            "article": "Article",
            "category_id": "GoodParentID",
            "kind_id": "ProductKindID",
            "producer_id": "ProducerID",
            "active_substance_id": "ActiveSubstanceID",
            "tax_id": "VATRate",
            "uktzed": "UCGFEA",
            "deletion_mark": "DeletionMark"
        },
        resolver: (item) => {
            return item.map(it => {
                it['producer_id'] = it['producer_id'] || null
                it['active_substance_id'] = it['active_substance_id'] || null
                return it
            })
        }
    },
    "products_barcodes": {
        "key": "Barcodes",
        "structure": {
            "product_id": "ProductID",
            "characteristic_id": "CharacteristicID",
            "packing_id": "PackageID",
            "barcode": "Barcode",
            "deletion_mark": "DeletionMark"
        }
    },
    "products_producers": {
        "key": "Producers",
        "structure": {
            "id": "ProducerID",
            "name": "Name",
            "deletion_mark": "DeletionMark"
        }
    },
    "products_active_substances": {
        "key": "ActiveSubstances",
        "structure": {
            "id": "ActiveSubstanceID",
            "name": "Name",
            "deletion_mark": "DeletionMark"
        }
    },
    "products_kinds": {
        "key": "ProductKinds",
        "structure": {
            "id": "ProductKindID",
            "parent_id": "ProductKindParentID",
            "name": "Name",
            "is_folder": "IsFolder",
            "use_series": "KeepAccountingBySeries",
            "use_characteristics": "UseCharacteristics",
            "deletion_mark": "DeletionMark"
        }
    },
    "products_categories": {
        "key": "Hierarchy",
        "structure": {
            "id": "ProductID",
            "parent_id": "GoodParentID",
            "name": "Name",
            "deletion_mark": "DeletionMark"
        },
        resolver: (items) => {
            return items.map(it => {
                it['parent_id'] = it['parent_id'] || null
                return it
            })
        }
    },
    "products_packings": {
        "key": "Packages",
        "structure": {
            "id": "PackageID",
            "product_id": "ProductID",
            "name": "Name",
            "deletion_mark": "DeletionMark"
        }
    },
    "series": {
        "key": "ProductSeries",
        "structure": {
            "id": "SeriesID",
            "product_id": "ProductID",
            "kind_id": "ProductKindID",
            "name": "Name",
            "number": "Number",
            "expiration_date": "ExpirationDate",
            "deletion_mark": "DeletionMark"
        }
    },
    "taxes": {
        "key": "Taxes",
        "structure": {
            "id": "TaxID",
            "value": "Value",
            "name": "Name",
            "code": "Code",
            "letter": "Letter",
            "deletion_mark": "DeletionMark"
        }
    },
    "price_types": {
        "key": "PriceTypes",
        "structure": {
            "id": "PriceTypeID",
            "name": "Name",
            "deletion_mark": "DeletionMark"
        }
    },
    "warehouses": {
        "key": "Warehouses",
        "structure": {
            "id": "WarehouseID",
            "name": "Name",
            "subdivision_id": "DepartmentID",
            "subdivision_name": "Department",
            "price_type_id": "PriceTypeID",
            "use_cells": "BinLocationWarehousing",
            "use_advanced_warehousing": "AdvancedWarehousingOnGoodsReceipt",
            "deletion_mark": "DeletionMark"
        },
        resolver: (items) => {
            return items.map(it => {
                it['price_type_id'] = it['price_type_id'] || null
                it['use_cells'] = !!it['use_cells']
                return it
            })
        }
    },
    "subdivisions": {
        "key": "Departments",
        "structure": {
            "id": "DepartmentID",
            "name": "Name",
            "deletion_mark": "DeletionMark"
        }
    },
    "counterparties": {
        "key": "Partners",
        "structure": {
            "id": "PartnerID",
            "fullname": "Fullname",
            "name": "Name",
            "email": "Email",
            "phone": "Phone",
            "is_client": "Client",
            "is_supplier": "Supplier",
            "deletion_mark": "DeletionMark"
        }
    },

    // >>> Документи <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    "arrivals": {
        "key": "GoodsAcceptance",
        "structure": {
            "id": "UUID",
            "number": "Number",
            "deletion_mark": "DeletionMark",
            "comment": "Comment",
            "createdAt": "Date",
            "warehouse_id": "WarehouseID",
            "price_type_id": "PriceTypeID",
            "counterparty_id": "CounterpartyID",
            "price_includes_vat": "PriceIncludesVAT",
            "goods": {
                "key": "Goods",
                "structure": {
                    "product_id": "ProductID",
                    "series_id": "SeriesID",
                    "packing_id": "PackageID",
                    "amount": "Quantity",
                    "total": "Amount"
                },
                resolver: (item) => {
                    return item.map(it => {
                        it['price'] = Round.money(it.total / it.amount)
                        delete it['total']
                        return it
                    })
                }
            }
        },
        resolver: (items) => {
            return items.map(it => {
                it['number'] = parseInt(it['number'].replace(/[a-zA-Zа-яА-Яії]/g, '')) || 0
                it['goods'] = it['goods'].map(good => {
                    good['document_id'] = it['id']
                    return good
                })
                return it
            })
        }
    },

    // >>> Регістри <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    "prices": {
        "key": "ProductPrices",
        "structure": {
            "recorder": {
                "key": "Recorder",
                "structure": {
                    "id": "ID",
                    "manager": "MANAGER"
                }
            },
            "items": {
                "key": "Records",
                "structure": {
                    "price_type_id": "PriceTypeID",
                    "product_id": "ProductID",
                    "series_id": "SeriesID",
                    "packing_id": "PackageID",
                    "price": "Price",
                    "period": "Period"
                }
            }
        },
        resolver: (item) => {
            return [null, null, ...item].map(it => it?.items?.map((item) => ({ ...item, basis_id: it.recorder.id }))).reduce((prev, next) => {
                return [...(prev || []), ...(next || [])]
            }).filter(it => !!it)
        }
    },

    "stocks": {
        "key": "GoodsInWarehouses",
        "structure": {
            "recorder": {
                "key": "Recorder",
                "structure": {
                    "id": "ID",
                    "manager": "MANAGER"
                }
            },
            "items": {
                "key": "Records",
                "structure": {
                    "warehouse_id": "WarehouseID",
                    "product_id": "ProductID",
                    "series_id": "SeriesID",
                    "assignment_id": "AssignmentID",
                    "in_stock": "InStock",
                    "to_ship": "ToShip",
                    "period": "Period"
                }
            }
        },
        resolver: (item) => {
            return [null, null, ...item].map(it => it?.items?.map((item) => ({ ...item, basis_id: it.recorder.id }))).reduce((prev, next) => {
                return [...(prev || []), ...(next || [])]
            }).filter(it => !!it)
        }
    },

}