import { Op } from '@sequelize/core'
import Database from '../database/index.js'
import ProductPriceRegisterValuesHelper from '../database/schemes/registers/product/product_price_values_helper.js'

export default class ArrivalHelper {

    static async ApplyProductsStockMoves({ basis_id, warehouse_id, assignment_id = "", period, items }, transaction) {
        // дата запису в регістер залишків
        const updatedAt = new Date().toISOString()
        // отримуємо товари
        const products = await Database.Products.findAll({
            attributes: ['id', 'name'],
            where: { id: { [Op.in]: items.map(it => it.product_id) } },
            raw: true,
            transaction
        })
        // отримуємо коефіцієнти для пакувань
        const packings_coeficients = await Database.ProductPacking.findAll({
            attributes: [['id', 'packing_id'], 'product_id', 'name', 'coefficient'],
            where: { id: { [Op.in]: items.map(it => it.packing_id) } },
            raw: true,
            transaction,
        })
        // помічаємо попередні записи регістру на вилучення
        await Database.Registers.ProductStock.update({ deletion_mark: true }, { where: { basis_id }, transaction })
        // готуємо записи у формат регістру залишків
        const records = items.map(item => {
            const packing = packings_coeficients.find(it => it.product_id === item.product_id && it.packing_id === item.packing_id)
            if (!packing) {
                const product = products.find(it => it.id === item.product_id)
                throw Error(`Не вдалось знайти пакування для товару ${product?.name ? `"${product.name}"` : ''}`)
            }
            if (!packing?.coefficient) {
                const product = products.find(it => it.id === item.product_id)
                throw Error(`Для товару ${product?.name ? `"${product.name}"` : ''} вказаний некоректний коефіцієнт пакування`)
            }
            return {
                basis_id,
                product_id: item.product_id,
                series_id: item.series_id,
                warehouse_id,
                assignment_id,
                in_stock: item.amount * packing?.coefficient,
                to_ship: 0,
                period
            }
        })
        for (let i = 0; i < records.length; i++) {
            // оновлюємо нові записи регістру і знімаємо мітку про вилучення
            await Database.Registers.ProductStock.upsert({ ...records[i], deletion_mark: false, updatedAt }, { transaction })
        }
    }

    static async ApplyProductsPriceMoves({ basis_id, price_type_id, period, items }, transaction) {
        // дата запису в регістер цін
        const updatedAt = new Date().toISOString()
        // помічаємо попередні записи регістру на вилучення
        await Database.Registers.ProductPrice.update({ deletion_mark: true }, { where: { basis_id } })
        // готуємо записи у формат регістру цін
        const records = items.map(it => ({ basis_id, product_id: it.product_id, series_id: it.series_id, packing_id: it.packing_id, price_type_id, price: it.price, period }))
        for (let i = 0; i < records.length; i++) {
            // оновлюємо нові записи регістру і знімаємо мітку про вилучення
            await Database.Registers.ProductPrice.upsert({ ...records[i], deletion_mark: false, updatedAt }, { transaction: transaction })
        }
        // перераховуємо актуальні ціни у службовій табличці
        await Promise.all(records.map(({ product_id, series_id, packing_id, price_type_id }) => (
            ProductPriceRegisterValuesHelper.recalculate({ where: { product_id, series_id, packing_id, price_type_id }, transaction })
        )))
    }
}