import { Op } from '@sequelize/core'
import Database from '../database/index.js'
import Round from '../lib/Round.js'

export default class InventoryHelper {

    static async UpdateDocumentGoods({ id, subdivision_id, updatedAt, items }, transaction) {
        // помічаємо попередні позиції на вилучення
        await Database.Documents.InventoryGoods.update({ deletion_mark: true }, { where: { document_id: id }, transaction })
        // підготовлюємо записи до формату бази даних
        const records = items.map((item, position) => ({ ...item, document_id: id, subdivision_id, position, deletion_mark: false, updatedAt }))
        // отримуємо існуючі записи по цьому документу
        const exists = await Database.Documents.InventoryGoods.findAll({ where: { document_id: id }, transaction })
        const should_create = []
        const should_update = []
        // функція яка повертає набір первинних ключів для запиту оновлення
        function keys({ subdivision_id, document_id, product_id, series_id, packing_id }) {
            return { subdivision_id, document_id, product_id, series_id, packing_id }
        }
        // функція яка повертає композитний ключ для пошуку позицій в переліку існуючих
        function hash({ subdivision_id, document_id, product_id, series_id, packing_id }) {
            return `${subdivision_id}-${document_id}-${product_id}-${series_id}-${packing_id}`
        }
        // фільтруємо записи на ті які потрібно оновити або створити
        for (let i = 0; i < records.length; i++) {
            if (exists.find(it => hash(it) === hash(records[i]))) {
                should_update.push(records[i])
            } else {
                should_create.push(records[i])
            }
        }
        if (should_update.length) await Promise.all(should_update.map(record => Database.Documents.InventoryGoods.update(record, { where: keys(record), transaction })))
        if (should_create.length) await Database.Documents.InventoryGoods.bulkCreate(should_create, { transaction })
    }

    static async ApplyProductsStockMoves({ basis_id, warehouse_id, assignment_id = "", period, items }, transaction) {
        // дата запису в регістер залишків
        const updatedAt = new Date().toISOString()
        // отримуємо товари
        const products = await Database.Products.findAll({
            attributes: ['id', 'name'],
            where: { id: { [Op.in]: items.map(it => it.product_id) } },
            raw: true,
            transaction
        })
        // отримуємо коефіцієнти для пакувань
        const packings_coeficients = await Database.ProductPacking.findAll({
            attributes: [['id', 'packing_id'], 'product_id', 'name', 'coefficient'],
            where: { id: { [Op.in]: items.map(it => it.packing_id) } },
            raw: true,
            transaction,
        })
        // помічаємо попередні записи регістру на вилучення
        await Database.Registers.ProductStock.update({ deletion_mark: true }, { where: { basis_id }, transaction })
        // готуємо записи у формат регістру залишків
        const records = items.map(item => {
            const packing = packings_coeficients.find(it => it.product_id === item.product_id && it.packing_id === item.packing_id)
            if (!packing) {
                const product = products.find(it => it.id === item.product_id)
                throw Error(`Не вдалось знайти пакування для товару ${product?.name ? `"${product.name}"` : ''}`)
            }
            if (!packing?.coefficient) {
                const product = products.find(it => it.id === item.product_id)
                throw Error(`Для товару ${product?.name ? `"${product.name}"` : ''} вказаний некоректний коефіцієнт пакування`)
            }
            return {
                basis_id,
                product_id: item.product_id,
                series_id: item.series_id,
                warehouse_id,
                assignment_id,
                in_stock: Round.quantity((item.actual_quantity - item.accounted_quantity) * packing?.coefficient),
                to_ship: 0,
                period
            }
        })
        // отримуємо існуючі записи регістру по цьому реєстратору
        const exists = await Database.Registers.ProductStock.findAll({ where: { basis_id }, transaction })
        const should_create = []
        const should_update = []
        // функція яка повертає набір первинних ключів для запиту оновлення
        function keys({ basis_id, product_id, series_id, warehouse_id, assignment_id }) {
            return { basis_id, product_id, series_id, warehouse_id, assignment_id }
        }
        // функція яка повертає композитний ключ для пошуку позицій в переліку існуючих
        function hash({ basis_id, product_id, series_id, warehouse_id, assignment_id }) {
            return `${basis_id}-${product_id}-${series_id}-${warehouse_id}-${assignment_id}`
        }
        // фільтруємо записи на ті які потрібно оновити або створити
        for (let i = 0; i < records.length; i++) {
            if (exists.find(it => hash(it) === hash(records[i]))) {
                should_update.push(records[i])
            } else {
                should_create.push(records[i])
            }
        }
        if (should_update.length) await Promise.all(should_update.map(record => Database.Registers.ProductStock.update(record, { where: keys(record), transaction })))
        if (should_create.length) await Database.Registers.ProductStock.bulkCreate(should_create, { transaction })
    }
}