import express from 'express'
import parser from 'body-parser'
import cors from 'cors'
import busboy from 'connect-busboy'
import compression from 'compression'
import fs from 'fs'
import http from 'http'
import https from 'https'
import { Server as SocketServer } from 'socket.io'
import SocketFactory from './lib/SocketFactory/index.js'

import moment from 'moment'
import 'moment/locale/uk.js'
moment.locale('uk')
console.clear()

import Database from './database/index.js'
import config from '../config/index.js'
import verify from './auth.js'

import AcquiringTerminalsReglament from './reglaments/acquiring.js'
import FiscalRegistersReglament from './reglaments/registers.js'
//import ScalesReglament from './reglaments/scales.js'

import ExchangeModule from './exchange/index.js'

import Replication from './replication/index.js'
Replication.start()

import Units from './models/dictionaries/units.js'
import Products from './models/dictionaries/products.js'
import Categories from './models/dictionaries/categories.js'
import ProductsPackings from './models/dictionaries/products_packings.js'
import ProductsBarcodes from './models/dictionaries/products_barcodes.js'
import ProductsShortcuts from './models/dictionaries/products_shortcuts.js'
import ProductsSeries from './models/dictionaries/products_series.js'


import Roles from './models/dictionaries/roles.js'
import Taxes from './models/dictionaries/taxes.js'
import Staff from './models/dictionaries/staff.js'
import Warehouses from './models/dictionaries/warehouses.js'
import PaymentsMethods from './models/dictionaries/payments_methods.js'
import Settings from './models/dictionaries/settings.js'


import Workplaces from './models/dictionaries/workplace.js'
import AcquiringTerminals from './models/dictionaries/acquiring_terminals.js'
import Scales from './models/dictionaries/scales.js'
import FiscalRegisters from './models/dictionaries/fiscal_registers.js'
import PrescriptionSuppliers from './models/dictionaries/prescription_suppliers.js'

import Shifts from './models/shifts/index.js'
import Transactions from './models/shifts/transactions.js'


import PriceTypes from './models/dictionaries/price_types.js'

import Documents from './models/documents/index.js'


import Subdivisions from './models/subdivisions.js'
import System from './models/system.js'

import ProductStockRegisterValuesHelper from './database/schemes/registers/product/product_stock_values_helper.js'
import ProductPriceRegisterValuesHelper from './database/schemes/registers/product/product_price_values_helper.js'
import Uploads from './lib/Uploads.js'
import Reports from './models/reports/index.js'
import Counterparties from './models/dictionaries/counterparties.js'
import TransactionsArticles from './models/dictionaries/transactions_articles.js'



const app = express()
app.use(cors({
	exposedHeaders: [
		'Pagination',
		'Pagination-Limit',
		'Pagination-Offset',
		'Pagination-Total',
		'Pagination-Page-Current',
		'Pagination-Page-Total',
		'Pagination-Order-By',
		'Pagination-Order-By-Desc',
	]
}))
app.use(busboy())
app.use(parser.urlencoded({ extended: true, limit: '1024mb' }));
app.use(parser.json({ limit: '1024mb' }));
app.use(compression())
app.use(verify());

app.get('/staff/signin', Staff.signin)
app.get('/staff/signin/adminpanel', Staff.adminpanel)
app.get('/staff/signout', Staff.signout)

app.get('/staff/manager/signin', Staff.signinManager)
app.get('/staff/manager/signout', Staff.signoutManager)

app.get('/staff/profile', Staff.profile)
app.get('/staff/cashier', Staff.cashier)
app.get('/staff', Staff.all)
app.get('/staff/:id', Staff.get)
app.post('/staff', Staff.create)
app.post('/staff/:id', Staff.update)
app.post('/staff/:id/reset', Staff.reset)
app.delete('/staff/:id', Staff.update)

app.get('/counterparties/autocomplete', Counterparties.autocomplete)
app.get('/counterparties', Counterparties.all)
app.get('/counterparties/:id', Counterparties.get)
app.get('/counterparties/:id/products', Counterparties.products)
app.post('/counterparties', Counterparties.create)
app.post('/counterparties/:id', Counterparties.update)
app.delete('/counterparties/:id', Counterparties.update)

app.get('/roles', Roles.all)
app.get('/roles/:id', Roles.get)
app.post('/roles', Roles.create)
app.post('/roles/:id', Roles.update)
app.delete('/roles/:id', Roles.delete)

app.get('/taxes', Taxes.all)
app.get('/taxes/autocomplete', Taxes.autocomplete)
app.get('/taxes/:id', Taxes.get)
app.post('/taxes', Taxes.create)
app.post('/taxes/:id', Taxes.update)
app.delete('/taxes/:id', Taxes.delete)

app.get('/units', Units.all)
app.get('/units/:id', Units.get)
app.post('/units', Units.create)
app.post('/units/:id', Units.update)
app.delete('/units/:id', Units.delete)

app.get('/price_types', PriceTypes.all)
app.get('/price_types/autocomplete', PriceTypes.autocomplete)
app.get('/price_types/:id', PriceTypes.get)
app.post('/price_types', PriceTypes.create)
app.post('/price_types/:id', PriceTypes.update)
app.delete('/price_types/:id', PriceTypes.delete)

app.get('/packings', ProductsPackings.all)
app.get('/packings/autocomplete', ProductsPackings.autocomplete)
app.get('/packings/:id', ProductsPackings.get)
app.post('/packings', ProductsPackings.create)
app.post('/packings/:id', ProductsPackings.update)
app.delete('/packings/:id', ProductsPackings.delete)

app.get('/barcodes', ProductsBarcodes.all)
app.get('/barcodes/autocomplete', ProductsBarcodes.autocomplete)
app.get('/barcodes/:barcode', ProductsBarcodes.get)
app.post('/barcodes', ProductsBarcodes.create)
app.post('/barcodes/:barcode', ProductsBarcodes.update)
app.delete('/barcodes/:barcode', ProductsBarcodes.delete)

app.get('/categories', Categories.all)
app.get('/categories/autocomplete', Categories.autocomplete)
app.get('/categories/:id', Categories.get)
app.post('/categories', Categories.create)
app.post('/categories/positions', Categories.positions)
app.post('/categories/:id', Categories.update)
app.delete('/categories/:id', Categories.delete)

app.get('/shortcuts', ProductsShortcuts.all)
app.get('/shortcuts/:id', ProductsShortcuts.get)
app.post('/shortcuts', ProductsShortcuts.create)
app.post('/shortcuts/positions', ProductsShortcuts.positions)
app.post('/shortcuts/:id', ProductsShortcuts.update)
app.delete('/shortcuts/:id', ProductsShortcuts.delete)

app.get('/products', Products.all)
app.get('/products/search', Products.search)
app.get('/products/barcode/:barcode', Products.barcode)
app.get('/products/substitutes/:id', Products.substitutes)
app.get('/products/find', Products.find)
app.get('/products/autocomplete', Products.autocomplete)
app.get('/products/:id', Products.get)
app.post('/products', Products.create)
app.post('/products/import', Products.import)
app.post('/products/:id', Products.update)
app.delete('/products/:id', Products.delete)

app.get('/series', ProductsSeries.all)
app.get('/series/autocomplete', ProductsSeries.autocomplete)
app.get('/series/:id', ProductsSeries.get)
app.post('/series', ProductsSeries.create)
app.post('/series/:id', ProductsSeries.update)
app.delete('/series/:id', ProductsSeries.delete)


app.get('/workplaces/login', Workplaces.login)
app.get('/workplaces/logout', Workplaces.logout)
app.get('/workplaces/current', Workplaces.current)
app.get('/workplaces', Workplaces.all)
app.get('/workplaces/:id', Workplaces.get)
app.post('/workplaces', Workplaces.create)
app.post('/workplaces/:id', Workplaces.update)
app.delete('/workplaces/:id', Workplaces.delete)

app.get('/devices/acquirings', AcquiringTerminals.all)
app.get('/devices/acquirings/autocomplete', AcquiringTerminals.autocomplete)
app.get('/devices/acquirings/:id', AcquiringTerminals.get)
app.post('/devices/acquirings', AcquiringTerminals.create)
app.post('/devices/acquirings/:id', AcquiringTerminals.update)
app.delete('/devices/acquirings/:id', AcquiringTerminals.delete)


app.get('/devices/scales/weight', Scales.weight)
app.get('/devices/scales', Scales.all)
app.get('/devices/scales/autocomplete', Scales.autocomplete)
app.get('/devices/scales/:id', Scales.get)
app.post('/devices/scales', Scales.create)
app.post('/devices/scales/:id', Scales.update)
app.delete('/devices/scales/:id', Scales.delete)

app.get('/devices/registers', FiscalRegisters.all)
app.get('/devices/registers/autocomplete', FiscalRegisters.autocomplete)
app.get('/devices/registers/:id', FiscalRegisters.get)
app.post('/devices/registers', FiscalRegisters.create)
app.post('/devices/registers/:id', FiscalRegisters.update)
app.delete('/devices/registers/:id', FiscalRegisters.delete)

app.get('/services/prescription/suppliers', PrescriptionSuppliers.all)
app.get('/services/prescription/suppliers/:id', PrescriptionSuppliers.get)
app.post('/services/prescription/suppliers', PrescriptionSuppliers.create)
app.post('/services/prescription/suppliers/:id', PrescriptionSuppliers.update)
app.delete('/services/prescription/suppliers/:id', PrescriptionSuppliers.delete)


app.get('/transactions/articles', TransactionsArticles.all)
app.get('/transactions/articles/autocomplete', TransactionsArticles.autocomplete)
app.get('/transactions/articles/:id', TransactionsArticles.get)
app.post('/transactions/articles', TransactionsArticles.create)
app.post('/transactions/articles/:id', TransactionsArticles.update)
app.delete('/transactions/articles/:id', TransactionsArticles.delete)

app.get('/transactions', Transactions.all)
app.get('/transactions/:id', Transactions.get)
app.post('/transactions', Transactions.create)

app.get('/shifts', Shifts.all)
app.get('/shifts/active', Shifts.active)
app.get('/shifts/active/balance', Shifts.balance)
app.get('/shifts/report', Shifts.report)
app.get('/shifts/report/register/x', Shifts.regsiterReportX)
app.get('/shifts/report/register/z', Shifts.regsiterReportZ)
app.get('/shifts/report/register/periodical', Shifts.regsiterReportPeriodical)

app.get('/shifts/:id', Shifts.get)
app.post('/shifts/open', Shifts.open)
app.post('/shifts/close', Shifts.close)



app.get('/payments/methods', PaymentsMethods.all)
app.get('/payments/methods/:id', PaymentsMethods.get)
app.post('/payments/methods', PaymentsMethods.create)
app.post('/payments/methods/positions', PaymentsMethods.positions)
app.post('/payments/methods/:id', PaymentsMethods.update)
app.delete('/payments/methods/:id', PaymentsMethods.delete)

app.get('/warehouses', Warehouses.all)
app.get('/warehouses/autocomplete', Warehouses.autocomplete)
app.get('/warehouses/:id', Warehouses.get)
app.post('/warehouses', Warehouses.create)
app.post('/warehouses/:id', Warehouses.update)
app.delete('/warehouses/:id', Warehouses.delete)

app.get('/documents/arrivals', Documents.Arrival.all)
app.get('/documents/arrivals/price', Documents.Arrival.price)
app.post('/documents/arrivals/history', Documents.Arrival.history)
app.get('/documents/arrivals/:id', Documents.Arrival.get)
app.get('/documents/arrivals/:id/pricing', Documents.Arrival.pricing)
app.get('/documents/arrivals/:id/return', Documents.Arrival.return)
app.post('/documents/arrivals', Documents.Arrival.create)
app.post('/documents/arrivals/:id', Documents.Arrival.update)
app.post('/documents/arrivals/:document_id/products/:product_id/series/:series_id', Documents.Arrival.add)
app.delete('/documents/arrivals/:document_id/products/:product_id/series/:series_id', Documents.Arrival.remove)

app.get('/documents/returns', Documents.Return.all)
app.get('/documents/returns/price', Documents.Return.price)
app.post('/documents/returns/history', Documents.Return.history)
app.get('/documents/returns/:id', Documents.Return.get)
app.post('/documents/returns', Documents.Return.create)
app.post('/documents/returns/:id', Documents.Return.update)
app.post('/documents/returns/:document_id/products/:product_id/series/:series_id', Documents.Return.add)
app.delete('/documents/returns/:document_id/products/:product_id/series/:series_id', Documents.Return.remove)

app.get('/documents/inventories', Documents.Inventory.all)
app.get('/documents/inventories/accounting', Documents.Inventory.accounting)
app.get('/documents/inventories/:id', Documents.Inventory.get)
app.post('/documents/inventories', Documents.Inventory.create)
app.post('/documents/inventories/:id', Documents.Inventory.update)

app.get('/documents/writeoff', Documents.Writeoff.all)
app.get('/documents/writeoff/:id', Documents.Writeoff.get)
app.post('/documents/writeoff', Documents.Writeoff.create)
app.post('/documents/writeoff/:id', Documents.Writeoff.update)

app.get('/documents/pricing', Documents.Pricing.all)
app.get('/documents/pricing/:id', Documents.Pricing.get)
app.post('/documents/pricing', Documents.Pricing.create)
app.post('/documents/pricing/:id', Documents.Pricing.update)

app.get('/documents/drafts', Documents.Draft.all)
app.get('/documents/drafts/:id', Documents.Draft.get)
app.post('/documents/drafts', Documents.Draft.create)
app.post('/documents/drafts/:id', Documents.Draft.update)
app.post('/documents/drafts/:id/product', Documents.Draft.product)
app.post('/documents/drafts/:id/cancel', Documents.Draft.cancel)

app.get('/documents/receipts', Documents.Receipt.all)
app.get('/documents/receipts/:id', Documents.Receipt.get)
app.post('/documents/receipts', Documents.Receipt.create)
app.post('/documents/receipts/sync', Documents.Receipt.sync)
app.post('/documents/receipts/:id', Documents.Receipt.update)
app.post('/documents/receipts/:id/pay', Documents.Receipt.pay)
app.post('/documents/receipts/:id/confirm', Documents.Receipt.confirm)
app.post('/documents/receipts/:id/restore', Documents.Receipt.restore)
app.post('/documents/receipts/:id/cancel', Documents.Receipt.cancel)
app.get('/documents/receipts/:id/return', Documents.Receipt.returnable)
app.post('/documents/receipts/:id/return', Documents.Receipt.return)
app.post('/documents/receipts/:id/withdrawal', Documents.Receipt.withdrawal)
app.get('/documents/receipts/:id/print', Documents.Receipt.print)

app.get('/reports/dashboard', Reports.Dashboard.get)
app.get('/reports/stock', Reports.Stock.get)
app.get('/reports/money', Reports.Money.get)
app.get('/reports/sales', Reports.Sales.get)


app.get('/settings', Settings.get)
app.post('/settings', Settings.update)

app.post('/uploads/:model/:id', Uploads.post)
app.get('/uploads/:model/:id/:file', Uploads.get)



app.get('/subdivisions', Subdivisions.all)
app.get('/subdivisions/current', Subdivisions.current)
app.get('/subdivisions/autocomplete', Subdivisions.autocomplete)
app.get('/subdivisions/:id/select', Subdivisions.change)
app.post('/subdivisions', Subdivisions.create)
if (config.replication.enabled && config.subdivision.is_master) {
	app.post('/replication', Replication.request)
}
if (config.exchange.enabled) {
	app.get('/exchange', ExchangeModule.send)
	app.post('/exchange', ExchangeModule.receive)
}


app.post('/update', System.autoupdate)
app.post('/update/:version', System.update)


app.post('/print', System.print)
app.get('/error/:id', System.error)

app.get('/', (req, resp) => resp.sendStatus(200))

app.get('*', System.unknown);
app.put('*', System.unknown);
app.post('*', System.unknown);
app.delete('*', System.unknown);



if (config.http.enabled) {
	http.createServer(app).listen(config.http.port, () => console.log(`[${new Date().toISOString()}] `, `HTTP сервер запущено на порті ${config.http.port}`));
}
if (config.https.enabled) {
	const key = fs.readFileSync(config.https.credentials.private_key, 'utf8');
	const cert = fs.readFileSync(config.https.credentials.certificate, 'utf8');
	const ca = fs.readFileSync(config.https.credentials.chain, 'utf8');
	https.createServer({ key, cert, ca }, app).listen(config.https.port, () => console.log(`[${new Date().toISOString()}] `, `HTTPS сервер запущено на порті ${config.https.port}`));
}

if (config.sockets.enabled) {
	const server = http.createServer(app).listen(config.sockets.port, () => console.log(`[${new Date().toISOString()}] `, `Sockets сервер запущено на порті ${config.sockets.port}`));
	const socket = new SocketServer(server, { cors: { origin: '*', methods: ["GET", "POST"] } });
	socket.on('connection', SocketFactory.onConnect);
	//socket.on('connection', s=>s.disconnect());
}
