import net from 'net'
import constaints from './constants/index.json' assert {type: "json"};
import vendors from './constants/vendors.json' assert {type: "json"};

const delay = (timeout) => new Promise((resolve) => setTimeout(resolve, timeout))


export default class Privat {

    static Error = class extends Error {
        constructor(message, code, canceled) {
            super(message)
            this.code = code
            this.canceled = !!canceled
        }
    }


    static get codes() {
        return constaints.codes
    }


    constructor(url, onDestroy, { logging }) {
        this.url = url
        this.client = new net.Socket();
        this.onDestroy = onDestroy
        this.logging = logging
    }

    info = null
    messages = []


    error({ errorDescription, params: { responseCode } }) {
        const vendor = vendors[this.info?.vendor]
        if (!vendor) return new Privat.Error(errorDescription, responseCode)
        const error = vendor.errors[errorDescription]
        const message = error && vendor.messages ? vendor.messages[error] : errorDescription
        const code = error && vendor.codes ? vendor.codes[error] : responseCode
        const isCanceled = vendor.codes.CANCELED_BY_USER === code
        return new Privat.Error(message, code, isCanceled)
    }

    request(data, timeout = 100000, forward_zero = false) {
        return new Promise((resolve, reject) => {
            setTimeout(() => { reject({ message: "Вичерпано час запиту до терміналу", timeout: true }) }, timeout)
            var client = new net.Socket();
            var body = typeof data === 'string' ? data : JSON.stringify(data)
            var response = null

            client.connect(2000, this.url, () => {
                client.write(Buffer.from([...(forward_zero ? [0] : []), ...Buffer.from(body, 'utf8'), 0]), "utf-8")
                if (this.logging) console.log(`[${new Date().toISOString()}] `, "\n", 'PRIVATBANK надіслано запит: ', body, "\n",);
            });

            client.on('data', (message) => {
                const json = message.toString("utf-8", 0, message.length - 1)
                response = JSON.parse(json)
                if (this.logging) console.log(`[${new Date().toISOString()}] `, "\n", 'PRIVATBANK отримано відповідь: ', json, "\n",);
                client.destroy();
            });

            client.on('error', (error) => {
                reject(error)
            });

            client.on('close', () => {
                resolve(response)
            });
        })
    }

    connect() {
        return new Promise((resolve, reject) => {
            var buffer = ""
            this.client.connect(2000, this.url, () => { this.isConnected = true; resolve() });
            this.client.on('data', (message) => {
                const decoded = message.toString("utf-8")
                if (decoded.charCodeAt(decoded.length - 1) === 0) {
                    if (buffer.length) {
                        this.messages.push(buffer + decoded.slice(0, decoded.length - 1))
                        buffer = ""
                    } else {
                        this.messages.push(decoded.slice(0, decoded.length - 1))
                    }
                } else {
                    buffer += decoded
                }
            });
            this.client.on('error', (error) => {
                if (this.logging) console.log(`[${new Date().toISOString()}] `, 'PRIVATBANK помилка веб-сокету: ', error)
                reject(error)
            });
            this.client.on('close', () => {
                if (this.logging) console.log(`[${new Date().toISOString()}] `, 'PRIVATBANK веб-сокет закрито')
                this.isConnected = false
                if (typeof this.onDestroy === 'function') this.onDestroy()
                reject("Втрачений зв'язок із еквайеренговим терміналом")
            });
        })
    }

    async send(data, timeout = 65000) {
        if (!this.isConnected) await this.connect()
        return new Promise(async (resolve, reject) => {
            var wait = true
            if (timeout) setTimeout(() => {
                wait = false
                reject(`Вичерпано час запиту до терміналу`);
            }, timeout)
            var body = typeof data === 'string' ? data : JSON.stringify(data)
            if (this.logging) console.log(`[${new Date().toISOString()}] `, "PRIVATBANK надіслано по веб-сокету:", body)
            this.client.write(Buffer.from([0, ...Buffer.from(body, 'utf8'), 0]), "utf-8")

            while (wait) {
                await delay(200)
                for (let i = 0; i < this.messages.length; i++) {
                    var json_correct_start = this.messages[i][0] === '{' || this.messages[i][0] === '['
                    var json_correct_end = this.messages[i][this.messages[i].length - 1] === '}' || this.messages[i][this.messages[i].length - 1] === ']'
                    if (json_correct_start && json_correct_end) {
                        const response = JSON.parse(this.messages[i])
                        if (response.method === data.method) {
                            resolve(response)
                            this.messages = [...this.messages.slice(0, i), ...this.messages.slice(i + 1)]
                            wait = false
                            break
                        }
                    }
                }
            }
        })
    }


    async ping() {
        const result = await this.request({ method: "PingDevice", step: 0 }, 5000, true)
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async identify() {
        const result = await this.request({ method: "ServiceMessage", step: 0, params: { msgType: "identify" } }, 5000)
        if (!result.error) {
            return this.info = result.params
        } else {
            throw this.error(result)
        }
    }



    async checkConnection() {
        const result = await this.send({ method: "CheckConnection", step: 0 })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async getTerminalInfo() {
        const result = await this.send({ method: "GetTerminalInfo", step: 0 })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async readBankCard() {
        const result = await this.send({ method: "ReadCardBank", step: 0 })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }


    async readDiscountCard() {
        const result = await this.send({ method: "ReadCardDiscount", step: 0 })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async getBalance() {
        const result = await this.send({ method: "GetBalance", step: 0, params: { merchantId: "0" } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }


    async getXReport() {
        const result = await this.send({ method: "Audit", step: 0, params: { merchantId: "0" } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async getZReport() {
        const result = await this.send({ method: "Verify", step: 0, params: { merchantId: "0" } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }




    async print(invoice) {
        const result = await this.send({ method: "PrintReceiptNum", step: 0, params: { invoiceNumber: invoice, prnFlag: "1" } }, 10000)
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }



    async getMerchantList() {
        const result = await this.send({ method: "ServiceMessage", step: 0, params: { msgType: "getMerchantList" } }, 3000)
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }




    async pay(amount) {
        const result = await this.send({ method: "Purchase", step: 0, params: { amount: amount.toFixed(2), discount: "", merchantId: 0, facepay: "false" } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async refund(amount, rrn) {
        const result = await this.send({ method: "Refund", step: 0, params: { amount: amount.toFixed(2), discount: "", merchantId: "0", rrn } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async withdrawal(invoice) {
        const result = await this.send({ method: "Withdrawal", step: 0, params: { invoiceNumber: invoice } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }

    async cancel() {
        const result = await this.send({ method: "ServiceMessage", step: 0, params: { msgType: "interrupt" } })
        if (!result.error) {
            return result.params
        } else {
            throw this.error(result)
        }
    }
}