import Checkbox from 'checkbox-prro'
import Excellio from 'exellio-rro'
import Maria from 'maria-rro'

import config from '../../../config/index.js'
import Database from '../../database/index.js'
import Receipts from '../../models/documents/receipts/index.js'
import SocketFactory from '../SocketFactory/index.js'

export default class FiscalRegistersFactory {
    static registers = []

    static async addRegister(id, options) {
        switch (options.vendor) {
            case "CHECKBOX":
                const has_offline_rceipt = await Database.Documents.Receipt.count({ where: { register_id: id, fiscal_sell_offline: true, fiscal_send: false } }) > 0
                const checkbox = new Checkbox({
                    api: "https://api.checkbox.in.ua/api/v1",
                    license: options.license,
                    has_offline_rceipt
                }, async (fiscal_code) => {
                    const used_by_transaction = await Database.Documents.ShiftsTransactions.findOne({ where: { fiscal_code } })
                    const used_by_receipt = await Database.Documents.Receipt.findOne({ where: { fiscal_code } })
                    return !used_by_transaction && !used_by_receipt
                })
                checkbox.onShiftChange(async (shift) => {
                    // send shift to client
                    console.log(`[${new Date().toISOString()}] `, 'Оновлення стану поточної зміни', shift?.serial, shift?.status)
                })
                checkbox.onRegisterChange(async (register) => {
                    const workplace = await Database.Workplaces.findOne({ where: { register_id } })
                    if (workplace) await SocketFactory.sendBalance(workplace.id)
                })
                if (options.token) {
                    checkbox.ajax.setToken(options.token)
                } else {
                    const error = await checkbox.signin(options.login, options.password)
                    if (!error) {
                        await Database.FiscalRegisters.update({ token: checkbox.token }, { where: { id: options.id } })
                    } else {
                        console.log(`[${new Date().toISOString()}] `, "CHECKBOX", "Помилка авторизації ", options.id)
                    }
                }
                FiscalRegistersFactory.registers.push({ id, instance: checkbox })
                return checkbox
            case "CASHALOT":
                break
            case "EXELLIO":
                const excellio = new Excellio({ ...options, logging: true })
                FiscalRegistersFactory.registers.push({ id, instance: excellio })
                await excellio.getShift()
                return excellio
            case "MARIA":
                const maria = new Maria({ ...options, logging: true })
                FiscalRegistersFactory.registers.push({ id, instance: maria })
                await maria.getShift()
                return maria
        }
    }

    static async getRegister(id) {
        try {
            const register = FiscalRegistersFactory.registers.find(it => it.id == id)?.instance
            if (!register) return null
            if (!register.isAuthorized) {
                error = await register.reaut()
                if (!register.isAuthorized || error) {
                    FiscalRegistersFactory.removeRegister(id)
                    return null
                }
            }
            return register
        } catch (error) {
            return null
        }
    }

    static removeRegister(id) {
        return FiscalRegistersFactory.registers = FiscalRegistersFactory.registers.filter(it => it.id != id)
    }

}