import { parse } from 'path'
import { SerialPort } from 'serialport'

function delay(timeout) { return new Promise((resolve) => setTimeout(resolve, timeout)) }

export default class Cas {

    constructor({ port, logging }) {
        this.port = port
        this.logging = logging
    }
    port = null
    logging = false
    connected = false
    data = null
    buffer = ""

    async connect(onClosed) {
        return await new Promise((resolve, reject) => {
            if (!this.port) reject('Не вказаний порт підключення ваги')
            this.serialport = new SerialPort({ path: this.port, baudRate: 9600 })
            this.serialport.on('open', (e) => {
                if (this.logging) console.log(`[${new Date().toISOString()}] `, `Відкрито зʼєднання з вагою CAS на порті ${this.port}`)
                this.connected = true
                this.buffer = ""
                resolve(null)
            })
            this.serialport.on('data', (data) => {
                const encoded = data.toString('utf-8')
                this.buffer += encoded
                console.log(`[${new Date().toISOString()}] `, `Дані від порту ${this.port}: |${this.buffer}|`)
                const parsed = /.*?([0-9\.]*)kg.*?/.exec(this.buffer)
                if (parsed && parsed.length >= 1) {
                    this.data = { weight: parsed[1] }
                    this.buffer = ""
                    console.log(`[${new Date().toISOString()}] `, `Вага на проті ${this.port}:`, this.data.weight)
                }
            })
            this.serialport.on('error', (error) => {
                if (this.logging || true) console.log(`[${new Date().toISOString()}] `, `Помилка ваги CAS на порті ${this.port}`, error)
                reject(error)
            })
            this.serialport.on('close', () => {
                if (this.logging) console.log(`[${new Date().toISOString()}] `, `Розірвано зʼєднання з вагою CAS на порті ${this.port}`)
                if (onClosed) onClosed()
                this.connected = false
                this.buffer = ""
            })
        })
    }

    async refresh() {
        return await new Promise(async (resolve, reject) => {
            this.data = null
            if (!this.connected) {
                await this.connect()
                if (!this.connected) return reject(Error("Відсутнє підключення до ваги"))
            }
            console.log(`[${new Date().toISOString()}] `, 'Запит на зчитування ваги')
            var attempt = 20
            while (attempt > 0) {
                this.serialport.write(Buffer.from([5]))
                this.serialport.write(Buffer.from([17]))
                await delay(100)
                if (this.data) {
                    resolve(parseFloat(this.data.weight))
                    this.data = null
                    return
                }
                attempt--
            }
            resolve(null)
        })
    }

    async check() {
        if (typeof await this.refresh() !== 'number') throw Error("Відсутнє підключення до ваги")
    }

    async close() {
        this.serialport.close()
    }
}