import config from "../../../config/index.js"
import Database from "../../database/index.js"
import Receipts from "../../models/documents/receipts/helper.js"
import Shifts from "../../models/shifts/index.js"
import FiscalRegistersFactory from "../FiscalRegistersFactory/index.js"
import JWT from "../JsonWebToken.js"


export default class SocketFactory {
    static clients = []

    static async onReceiveMessage(sender, type, data) {
        switch (type) {
            case 'notification_read':
                await Database.System.Notifications.update({ is_new: false }, { where: { id: data.id } })
                await SocketFactory.sendNotification(sender.id, data.id)
                break;
        }
    }

    static onConnect(socket) {
        try {
            const auth = JWT.verify(socket.handshake.auth.token)
            if (!auth) return socket.disconnect()
            SocketFactory.clients.push({ id: socket.id, instance: socket, ...auth })
            console.log(`[${new Date().toISOString()}] `, 'Веб-сокет підʼєднано', socket.id, auth.workplace_id)
            SocketFactory.sendBalance(socket.id)
            SocketFactory.sendNotifications(socket.id)
            socket.on("disconnect", () => {
                SocketFactory.clients = SocketFactory.clients.filter(it => it.id !== socket.id)
                console.log(`[${new Date().toISOString()}] `, 'Веб-сокет втратив зʼєднання', socket.id, auth.workplace_id)
            });
            socket.on("message", ({ type, data }) => SocketFactory.onReceiveMessage(socket, type, data))
        } catch (error) {
            console.log("Помилка підключення сокету", error)
        }
    }

    static send(id, type, data) {
        const socket = SocketFactory.clients.find(client => client.user_id === id || client.workplace_id === id)
        if (!socket || !socket?.instance) return console.log(`[${new Date().toISOString()}] `, "Веб-сокет не знайдено", id)
    }

    static async sendBalance(id) {
        try {
            const socket = SocketFactory.clients.find(client => client.id === id || client.user_id === id || client.workplace_id === id || client.subdivision_id === id)
            if (!socket) return
            const { instance, workplace_id, subdivision_id } = socket
            const balance = await Shifts.prepareBalance(subdivision_id, workplace_id)
            if (!balance) throw Error('Не вдалось підготувати дані для відправки')
            if (instance) instance.send({ type: 'balance', data: balance })
        } catch (error) {
            await Database.onCatchError(error, { name: 'Помилка при надсиланні балансу по веб-сокету', subdivision_id: config.subdivision.id })
        }
    }

    static async sendNotifications(id) {
        try {
            const sockets = SocketFactory.clients.filter(client => client.id === id || client.user_id === id || client.workplace_id === id || client.subdivision_id === id)
            for (let i = 0; i < sockets.length; i++) {
                const { instance, subdivision_id } = sockets[i]
                const notifications = await Database.System.Notifications.findAll({ where: { subdivision_id }, order: [['createdAt', 'DESC']], limit: 20 })
                if (!notifications) throw Error('Не вдалось підготувати дані для відправки')
                if (instance) instance.send({ type: 'notifications', data: notifications.map(row => row.get()) })
            }
        } catch (error) {
            await Database.onCatchError(error, { name: 'Помилка при надсиланні нотифікацій по веб-сокету', subdivision_id: config.subdivision.id })
        }
    }
    static async sendNotification(id, notification_id) {
        try {
            const sockets = SocketFactory.clients.filter(client => client.id === id || client.user_id === id || client.workplace_id === id || client.subdivision_id === id)
            for (let i = 0; i < sockets.length; i++) {
                const { instance, subdivision_id } = sockets[i]
                const notification = await Database.System.Notifications.findOne({ where: { subdivision_id, id: notification_id } })
                if (!notification) throw Error('Не вдалось підготувати дані для відправки')
                if (instance) instance.send({ type: 'notification_updated', data: notification.get() })
            }
        } catch (error) {
            await Database.onCatchError(error, { name: 'Помилка при надсиланні нотифікації по веб-сокету', subdivision_id: config.subdivision.id })
        }
    }

}