import config from "../../config/index.js";

import fetch from 'node-fetch'
import fs from "fs";
import unzipper from "unzipper";
import mergedirs from 'merge-dirs'
import { exec } from 'child_process'
import semver from "semver";

const api = 'http://mobile.medias.com.ua:3080/v2/update'
const application = 'com.medias.prodaj.api'
const root = config.app.path


export default class Updates {

    static async check(current = config.app.version) {
        try {
            console.log(`[${new Date().toISOString()}] `, 'Пошук оновлень розпочато...');
            if (!config.app.can_update) return null
            const response = await fetch(`${api}?app=${encodeURIComponent(application)}&version=${encodeURIComponent(current)}`)
            const result = await response.text()
            switch (response.status) {
                case 200:
                    console.log(`[${new Date().toISOString()}] `, `Доступні оновлення ${result}`);
                    const { latest, minor, patch, build } = JSON.parse(result)
                    return { latest, minor, patch, build }
                case 204: return null
                default: throw Error(result);
            }
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, `Не вдалось знайти оновлення: ${error.message}`);
            throw Error(`Не вдалось знайти оновлення: ${error.message}`)
        }
    }

    static async download(version = "1.0.0") {
        try {
            console.log(`[${new Date().toISOString()}] `, 'Завантаження оновлення розпочато...');
            if (semver.gt(config.app.version, version)) throw Error('Заборонено оновлення на нижчу версію за поточну')
            const response = await fetch(`${api}/${application}/${version}`)
            switch (response.status) {
                case 200:
                    await fs.promises.mkdir(`${root}/updates`, { recursive: true })
                    const stream = fs.createWriteStream(`${root}/updates/update.zip`);
                    return await new Promise((resolve, reject) => {
                        stream.on("finish", () => resolve(true));
                        response.body.on("error", reject);
                        response.body.pipe(stream);
                    });
                default: throw Error(message);
            }
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, `Не вдалось завантажити оновлення: ${error.message}`);
            throw Error(`Не вдалось завантажити оновлення: ${error.message}`)
        }
    }

    static async unzip() {
        try {
            const zip = fs.createReadStream(`${root}/updates/update.zip`).pipe(unzipper.Parse({ forceStream: true }));
            for await (const entry of zip) {
                switch (entry.type) {
                    case 'Directory': await fs.mkdir(`${root}/updates/${entry.path}`, { recursive: true }); break;
                    case 'File': entry.pipe(fs.createWriteStream(`${root}/updates/${entry.path}`)); break;
                    default: break;
                }
            }
            return await new Promise((resolve, reject)=>{
                fs.rm(`${root}/updates/update.zip`,{ force:true },()=>{setTimeout(()=>resolve(),1000)})
            })
           
        } catch (error) {
            console.error(`Не вдалось видобути оновлення: ${error.message}`);
            throw Error(`Не вдалось видобути оновлення: ${error.message}`)
        }
    }

    static async install() {
        try {
            mergedirs.default(`${root}/updates`, root, 'overwrite');
            return await new Promise((resolve, reject)=>{
                fs.rm(`${root}/updates`,{ recursive:true, forse:true },()=>{setTimeout(()=>resolve(),1000)})
            })
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, `Не вдалось встановити оновлення: ${error.message}`);
            throw Error(`Не вдалось встановити оновлення: ${error.message}`);
        }
    }

    static async restart() {
        try {
            exec(`pm2 restart ${config.pm2.name}`, (error, stdout, stderr) => {
                if (error) {
                    console.log(`[${new Date().toISOString()}] `, `Не вдалось перезапустити сервер: ${error.message}`);
                    return;
                }
                if (stderr) {
                    console.log(`[${new Date().toISOString()}] `, `Сервер успішно перезапущено`);
                    return;
                }
            });
            return true
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, `Не вдалось перезапустити сервіс: ${error.message}`);
            throw Error(`Не вдалось перезапустити сервіс: ${error.message}`);
        }
    }

    static async clean() {
        try {
            fs.rmSync(`${root} / updates`, { recursive: true, force: true });
        } catch (error) {
            console.log(`[${new Date().toISOString()}]`, `Не вдалось очистити директорію оновлень`);
        }
    }

}