import { v4 as uuid } from 'uuid'
import path from 'path'
import config from '../../config/index.js'
import fs from 'fs'

class Uploads {

    static storage = config.uploads.path //path.join(path.dirname(module.path), config.uploads.path)

    static save(model, basis_id, extention, data) {
        const upload_id = uuid()
        const folder = path.join(Uploads.storage, `/${model}/${basis_id}`)
        fs.mkdirSync(folder, { recursive: true })
        const destination = path.join(folder, `/${upload_id}.${extention}`)
        if (data.includes(';base64,')) {
            fs.writeFileSync(destination, data.split(';base64,')[1], { encoding: 'base64' })
        } else {
            fs.writeFileSync(destination, data, { encoding: 'base64' })
        }
        return `${config.uploads.url}/${model}/${basis_id}/${upload_id}.${extention}`
    }

    static filepath(model, basis_id, file) {
        const filepath = path.join(Uploads.storage, `/${model}/${basis_id}/${file}`)
        if (!fs.existsSync(filepath)) return null
        return fs.realpathSync(filepath)
    }

    static extension(file) {
        switch (file) {
            case 'image/png': return 'png'
            case 'image/bmp': return 'bmp'
            case 'image/jpeg': return 'jpeg'
            default:
                return /^data:image\/(.*?);base64,/.exec(file)?.at(1)
        }
    }

    static async post(req, res) {
        try {
            const { model, id } = req.params
            const { extension, type, data } = req.body
            const url = Uploads.save(model, id, extension || Uploads.extension(type) || Uploads.extension(data) || 'jpeg', data)
            res.status(200).send(JSON.stringify({ url }))
        } catch (error) {
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }

    static async get(req, res) {
        try {
            const { model, id, file } = req.params
            const filepath = Uploads.filepath(model, id, file)
            if (!filepath) return res.status(404).send()
            res.status(200).sendFile(filepath)
        } catch (error) {
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }


}

export default Uploads