import config from '../../../config/index.js';
import Database from '../../database/index.js'

export default class Settings {

    static prepare(data, subdivision_id) {
        return Object.keys(data).map(key => ({
            subdivision_id,
            key,
            type: typeof data[key],
            value: data[key].toString()
        })).filter(it => ['boolean', 'number', 'string'].includes(it.type))
    }

    static parse(records) {
        const data = {}
        records.map(record => {
            switch (record.type) {
                case 'boolean':
                    data[record.key] = record.value === 'true' || record.value === '1'; break;
                case 'number':
                    data[record.key] = parseFloat(record.value); break;
                default:
                    data[record.key] = record.value; break;
            }
        })
        return data
    }

    static async all(subdivision_id, version) {
        const saved = await Database.Settings.findAll({ where: { subdivision_id }, attributes: { exclude: ['subdivision_id'] }, order: [['key']] })
        const custom = [
            { key: 'api_version', type: 'string', value: version },
            { key: 'is_master', type: 'boolean', value: JSON.stringify(!!config.is_master) }
        ]
        return Settings.parse([...saved, ...custom])
    }

    static async get(req, res) {
        res.setHeader('Content-Type', 'application/json')
        try {
            const { version } = req.app
            const { subdivision_id } = req.auth
            res.status(200).send(JSON.stringify(await Settings.all(subdivision_id, version)))
        } catch (error) {
            await Database.onCatchError(error, { name: 'Get settings', subdivision_id: req.auth.subdivision_id })
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }

    static async update(req, res) {
        const transaction = await Database.instance.transaction()
        res.setHeader('Content-Type', 'application/json')
        try {
            const { version } = req.app
            const { subdivision_id } = req.auth
            const unsavable = ['api_version', 'is_master']
            const keys = Object.keys(req.body).filter(key => ['string', 'number', 'boolean'].includes(typeof req.body[key]) && !unsavable.includes(key))
            const rows = Settings.prepare(req.body, subdivision_id).filter(row => keys.includes(row.key))
            await Promise.all(rows.map(row => Database.Settings.upsert(row, { transaction })))
            await transaction.commit()
            res.status(200).send(JSON.stringify(await Settings.all(subdivision_id, version)))
        } catch (error) {
            await transaction.rollback()
            await Database.onCatchError(error, { name: 'Update settings', subdivision_id: req.auth.subdivision_id })
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }

}