import { Op } from '@sequelize/core'
import { v4 as uuid } from 'uuid'
import Database from '../../database/index.js'

export default class Units {

    static prepare(data, id) {
        const { name, parent, color, image } = data
        return ({
            id: id ? id : uuid(),
            name: name,
        })
    }

    static options(where = {}, params = {}) {
        return {
            where,
            order: [['createdAt', 'ASC']],
            ...params
        }
    }

    static async all(req, res) {
        res.setHeader('Content-Type', 'application/json')
        const transaction = await Database.instance.transaction()
        try {
            const where = Rest.where(req)
            const pagination = Rest.pagination(req, res, await Database.Units.count({ where, transaction }))
            const result = await Database.Units.findAll(Units.options(where, { ...pagination, transaction }))
            await transaction.commit()
            res.status(200).send(JSON.stringify(result))
        } catch (error) {
            await transaction.rollback()
            await Database.onCatchError(error, { name: 'All units', subdivision_id: req.auth.subdivision_id })
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }

    static async get(req, res) {
        res.setHeader('Content-Type', 'application/json')
        const transaction = await Database.instance.transaction()
        try {
            const { id } = req.params
            const result = await Database.Units.findOne(Units.options({ id }, { transaction }))
            await transaction.commit()
            res.status(200).send(JSON.stringify(result))
        } catch (error) {
            await transaction.rollback()
            await Database.onCatchError(error, { name: 'Get units', subdivision_id: req.auth.subdivision_id })
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }

    static async create(req, res) {
        res.setHeader('Content-Type', 'application/json')
        const transaction = await Database.instance.transaction()
        try {
            if (Array.isArray(req.body)) {
                const created = await Database.Units.bulkCreate(req.body.map(item => Units.prepare(item, item.id)), { transaction })
                const result = await Database.Units.findAll({ id: { [Op.in]: created.map(it => it.id) } }, { transaction })
                await transaction.commit()
                res.status(200).send(JSON.stringify(result))
            } else {
                const created = await Database.Units.create(Units.prepare(req.body, req.body.id))
                const result = await Database.Units.findAll({ id: created.id }, { transaction })
                await transaction.commit()
                res.status(200).send(JSON.stringify(result))
            }
        } catch (error) {
            transaction.rollback()
            await Database.onCatchError(error, { name: 'Create units', subdivision_id: req.auth.subdivision_id })
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }


    static async update(req, res) {
        res.setHeader('Content-Type', 'application/json')
        const transaction = await Database.instance.transaction()
        try {
            const { id } = req.params
            await Database.Units.update(Units.prepare(req.body, id), { where: { id }, transaction })
            const result = await Database.Units.findAll({ id }, { transaction })
            await transaction.commit()
            res.status(200).send(JSON.stringify(result))
        } catch (error) {
            await transaction.rollback()
            await Database.onCatchError(error, { name: 'Update units', subdivision_id: req.auth.subdivision_id })
            res.status(500).send(JSON.stringify({ message: error.message }))
        }
    }

    static async delete(req, res) {
        res.setHeader('Content-Type', 'application/json')
        try {
            const { id } = req.params
            await Database.Units.destroy({ where: { id } })
            res.status(200).send(JSON.stringify({ id }))
        } catch (error) {
            await Database.onCatchError(error, { name: 'Delete units', subdivision_id: req.auth.subdivision_id })
            switch (error.name) {
                case 'SequelizeForeignKeyConstraintError':
                    return res.status(500).send(JSON.stringify({ message: "Неможливо видалити" + "\n" + "Одиниця вимірювання використовується" }))
                default:
                    return res.status(500).send(JSON.stringify({ message: error.message }))
            }
        }
    }

}