
import { v4 as uuid } from 'uuid'
import sequelize, { Op } from '@sequelize/core'
import moment from 'moment'
import Database from '../../../database/index.js'
import Round from '../../../lib/Round.js'

export default class Hepler {

    static prepare(data, id) {
        const { subdivision_id, staff_id, number, presentation, status, workplace_id } = data
        return ({
            id: id ? id : uuid(),
            subdivision_id,
            number,
            presentation: presentation || `Підбір у чек від ${moment().format('DD.MM.YYYY')}`,
            status: status || `NEW`,
            staff_id,
            workplace_id,
        })
    }

    static prepareGood(data, id) {
        const { subdivision_id, document_id, product_id, amount, deletion_mark = false, position = 0, } = data
        return ({
            subdivision_id,
            id: id ? id : uuid(),
            document_id,
            product_id,
            series_id,
            warehouse_id,
            amount,
            deletion_mark,
            position,
        })
    }



    static query(where = {}, params = {}) {
        return {
            ...params,
            attributes: {
                exclude: ['subdivision_id', 'workplace_id', 'staff_id', 'shift_id']
            },
            include: [
                { model: Database.Workplaces, as: 'workplace', attributes: ["id", "name"] },
                { model: Database.Staff, as: 'staff', attributes: ["id", "firstname", "lastname"] },
                {
                    model: Database.Documents.DraftGoods, as: 'goods',
                    attributes: {
                        exclude: ['subdivision_id', 'document_id', 'deletion_mark', 'position', 'updatedAt']
                    },
                    include: [
                        { model: Database.Products, as: 'product', attributes: { exclude: ['subdivision_id', 'createdAt', 'updatedAt'] } },
                        { model: Database.ProductPacking, as: 'packing', attributes: { exclude: ['subdivision_id', 'createdAt', 'updatedAt'] } }
                    ],
                    required: false,
                    where: { 'deletion_mark': { [Op.or]: { [Op.is]: null, [Op.eq]: false } } }
                }
            ],
            where,
            order: [
                ['createdAt', 'DESC'],
                [{ model: Database.Documents.DraftGoods, as: 'goods' }, 'position', 'DESC']
            ],
        }
    }

    static async getOne(where, params) {
        var result = await Database.Documents.Draft.findOne(Hepler.query(where, params))
        var draft = result ? result.get({ plain: true }) : null
        if (draft) {
            draft.goods.map(good => good.total = Round.money(good.amount * good.price))
            draft['total'] = draft.goods.length ? Round.money(draft.goods.map(it => it.total).reduce((a, b) => a + b)) : 0
        }
        return draft
    }

    static async getAll(where = null, params) {
        const result = await Database.Documents.Draft.findAll(Hepler.query(where, params))
        return result.map(row => {
            let draft = row.get({ plain: true })
            draft.goods.map(good => good.total = Round.money(good.amount * good.price))
            draft['total'] = draft.goods.length ? Round.money(draft.goods.map(it => it.total).reduce((a, b) => a + b)) : 0
            return draft
        })
    }

}