import config from "../../config/index.js";
import Database from "../database/index.js";
import SocketFactory from "../lib/SocketFactory/index.js";
import Updates from "../lib/Updates.js";

//import printer from '@thiagoelg/node-printer'
export default class System {

    static async autoupdate(req, res) {
        res.setHeader('Content-Type', 'application/json')
        try {
            const available = await Updates.check()
            if (available?.build) {
                await Updates.download(available.build)
                await Updates.unzip()
                await Updates.install()
                res.status(200).send(JSON.stringify({ message: 'Сервіс успішно оновлено', status: true, error: false, version: available?.build }))
                await Updates.restart()
            } else {
                res.status(200).send(JSON.stringify({ message: 'Не знайдено доступних оновлень', status: false, error: false, version: config.app.version }))
            }
        } catch (error) {
            await Updates.clean()
            console.log(`[${new Date().toISOString()}] `, error.message)
            res.status(500).send(JSON.stringify({ message: error.message, status: false, error: true }))
        }
    }

    static async update(req, res) {
        res.setHeader('Content-Type', 'application/json')
        try {
            const { version } = req.params
            if (config.app.version === version) return res.status(200).send(JSON.stringify({ message: 'Сервіс не потребує оновлення', status: true, error: false }))
            const available = await Updates.check(version)
            if (!available.build) return res.status(500).send(JSON.stringify({ message: 'Не знайдено потрібної версії для оновлення', status: true, error: true }))
            await Updates.download(available.build)
            await Updates.unzip()
            await Updates.install()
            res.status(200).send(JSON.stringify({ message: 'Сервіс успішно оновлено', status: true, error: false }))
            await Updates.restart()
        } catch (error) {
            await Updates.clean()
            console.log(`[${new Date().toISOString()}] `, error.message)
            res.status(500).send(JSON.stringify({ message: error.message, status: false }))
        }
    }


    static async print(req, res) {
        try {
            res.setHeader('Content-Type', 'application/json')
            const { base64 } = req.body

            const image = Buffer.from(base64, "base64");

            // printer.printDirect({
            //     data: image
            //     , success: function (id) {
            //         return res.status(200).send(JSON.stringify({ id }))
            //     }
            //     , error: function (err) {
            //         return res.status(500).send(JSON.stringify({ message: err.message }))
            //     }
            // });

        } catch (error) {
            res.status(500).send(JSON.stringify({ message: error.message, canceled: error.canceled }))
        }
    }

    static async error(req, res) {
        try {
            res.setHeader('Content-Type', 'application/json')
            const { workplace_id } = req.auth
            const { id } = req.params
            const { notification_id } = req.query
            const error = await Database.System.Errors.findOne({ where: { id } })
            if (!error) {
                res.status(404).send(JSON.stringify({ message: 'Дані про цю помилку не знайдено' }))
            } else {
                if (notification_id) {
                    await Database.System.Notifications.update({ is_new: false }, { where: { id: notification_id } })
                    await SocketFactory.sendNotifications(workplace_id)
                }
                res.status(200).send(error.stack)
            }
        } catch (error) {
            res.status(500).send(JSON.stringify({ message: error.message, canceled: error.canceled }))
        }
    }

    static async unknown(req, res) {
        try {
            res.setHeader('Content-Type', 'application/json')
            res.status(404).send(JSON.stringify({ message: 'Не знайдено методу по цій адресі' }))
        } catch (error) {
            res.status(500).send(JSON.stringify({ message: error.message, canceled: error.canceled }))
        }
    }
}