import { CronJob } from 'cron'
import AquaringTerminalsFactory from '../lib/AquaringTerminalsFactory/index.js'
import config from '../../config/index.js'
import Database from '../database/index.js'

async function AcquiringTerminalsReglament() {
    const terminals = await Database.AcquiringTerminals.findAll({ where: { is_enabled: true, subdivision_id: config.subdivision.id } }).then(rows => rows.map(row => row.get({ plain: true })))
    for (let i = 0; i < terminals.length; i++) {
        try {
            var terminal = AquaringTerminalsFactory.getTerminal(terminals[i].id)
            if (!terminal) {
                switch (terminals[i].vendor) {
                    case 'PRIVATBANK':
                        terminal = await AquaringTerminalsFactory.addTerminal(terminals[i].id, terminals[i].vendor, terminals[i])
                        break;
                    default:
                        throw Error("Екваєренговий термінал не підтримується")
                        break;
                }
            }
            const response = await terminal.checkConnection()
            //console.log(`[${new Date().toISOString()}] `, 'Aquiring connection state', response)
            await Database.AcquiringTerminals.update({ status: 1, error: '' }, { where: { id: terminals[i].id } })
        } catch (error) {
            //console.log(`[${new Date().toISOString()}] `, "\n\n", 'ERROR: Can`t connect to terminal (' + terminals[i].id + ') ', error.message, "\n\n")
            AquaringTerminalsFactory.removeTerminal(terminals[i].id)
            await Database.AcquiringTerminals.update({ status: 0, error: error.message }, { where: { id: terminals[i].id } })
        }
    }
}


var AcquiringStatusReglament = new CronJob('0 * * * * *', AcquiringTerminalsReglament, null, true);

export default { AcquiringStatusReglament }