import { CronJob } from 'cron'
import FiscalRegistersFactory from '../lib/FiscalRegistersFactory/index.js'
import config from '../../config/index.js'
import Database from '../database/index.js'
import Receipts from '../models/documents/receipts/index.js'
import SocketFactory from '../lib/SocketFactory/index.js'
import Notifications from '../lib/Notifications.js'

const subdivision_id = config.subdivision.id

async function FiscalRegistersReglament() {
    const registers = await Database.FiscalRegisters.findAll({ where: { is_enabled: true, subdivision_id } }).then(rows => rows.map(row => row.get({ plain: true })))
    if (registers?.length) {
        console.log(`[${new Date().toISOString()}]`, 'Перевірка статусу реєстраторів')
        for (let i = 0; i < registers.length; i++) {
            try {
                var register = await FiscalRegistersFactory.getRegister(registers[i].id)
                if (!register) {
                    register = await FiscalRegistersFactory.addRegister(registers[i].id, registers[i])
                }
                const error_message = await register.getRegisterInfo()
                if (error_message) {
                    throw Error(error_message)
                } else {
                    await Database.FiscalRegisters.update({ status: 1, error: '' }, { where: { id: registers[i].id } })
                }
            } catch (error) {
                console.log(`[${new Date().toISOString()}]`, 'Неможливо підʼєднатись до РРО', registers[i].id, error.message)
                const error_id = await Database.onCatchError(error, { name: `Неможливо підʼєднатись до РРО ${registers[i].id.split('-')[0]}`, subdivision_id: config.subdivision.id })
                await Notifications.create({ type: 'error', text: `Неможливо підʼєднатись до РРО ${registers[i].id.split('-')[0]}`, error_id })
                await Database.FiscalRegisters.update({ status: 0, error: error.message }, { where: { id: registers[i].id } })
            }
        }
    }
}


async function FiscalRegistersSyncReglament() {
    const registers = await Database.FiscalRegisters.findAll({ where: { is_enabled: true, subdivision_id } }).then(rows => rows.map(row => row.get({ plain: true })))
    for (let i = 0; i < registers.length; i++) {
        try {
            var register = await FiscalRegistersFactory.getRegister(registers[i].id)
            if (register) {
                const error = await register.getRegisterInfo()
                if (!error) {
                    if (await Receipts.hasOfflineReceipts(registers[i].id)) {
                        console.log(`[${new Date().toISOString()}]`, 'Розпочато надсилання офлайн чеків у ДПС')
                        await Receipts.syncOfflineReceipts(registers[i].id)
                        register.has_offline_rceipt = await Receipts.hasOfflineReceipts(registers[i].id)
                        const workplace = await Database.Workplaces.findOne({ where: { register_id: registers[i].id } })
                        if (workplace) await SocketFactory.sendBalance(workplace.id)
                    } else {
                        const workplace = await Database.Workplaces.findOne({ where: { register_id: registers[i].id } })
                        if (workplace) await SocketFactory.sendBalance(workplace.id)
                    }
                }
            }
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, 'Помилка надсилання офлайн чеків у ДПС', registers[i].id, error)
            const error_id = await Database.onCatchError(error, { name: `Помилка надсилання офлайн чеків у ДПС ${registers[i].id.split('-')[0]}`, subdivision_id: config.subdivision.id })
            await Notifications.create({ type: 'error', text: `Помилка надсилання офлайн чеків у ДПС ${registers[i].id.split('-')[0]}`, error_id })
        }
    }
}


var RegisterStatusReglament = new CronJob('0 */1 * * * *', FiscalRegistersReglament, null, true);
var RegisterSyncReglament = new CronJob('0 */5 * * * *', FiscalRegistersSyncReglament, null, false);

export default { RegisterStatusReglament, RegisterSyncReglament }

