import ScaleFactory from '../lib/ScaleFactory/index.js'
import config from '../../config/index.js'
import Database from '../database/index.js'

async function ScalesReglament() {
    const scales = await Database.Scales.findAll({ where: { is_enabled: true, subdivision_id: config.subdivision.id } }).then(rows => rows.map(row => row.get({ plain: true })))
    for (let i = 0; i < scales.length; i++) {
        try {
            var scale = ScaleFactory.getScale(scales[i].id)
            if (!scale) {
                switch (scales[i].vendor) {
                    case 'CAS':
                        scale = await ScaleFactory.addScale(scales[i].id, scales[i].vendor, scales[i])
                        break;
                    default:
                        throw Error(`Вага "${scales[i].vendor}" не підтримується`)
                        break;
                }
            }
            await scale.check()
            await Database.Scales.update({ status: 1, error: '' }, { where: { id: scales[i].id } })
        } catch (error) {
            console.log(`[${new Date().toISOString()}] `, 'Не вдається підключитись до ваги (' + scales[i].id + ') ', error)
            ScaleFactory.removeScale(scales[i].id)
            await Database.Scales.update({ status: 0, error: error.message }, { where: { id: scales[i].id } })
        }
    }
}

export default setInterval(ScalesReglament, 60 * 1000)