import { Op, Transaction } from '@sequelize/core'
import Database from '../database/index.js'
import ProductStockRegisterValuesHelper from '../database/schemes/registers/product/product_stock_values_helper.js'
import ProductPriceRegisterValuesHelper from '../database/schemes/registers/product/product_price_values_helper.js'

/*  Перелік даних у пакеті:
    1) Ролі користувачів
    2) Податкові ставки
    3) Статті транзакцій
    4) Категорії товарів
    5) Види товарів
    6) Пакування товарів
    7) Характеристики товарів
    8) Виробники товарів
    9) Активні речовини товарів
    10) Товари
    11) Штрихкоди
    12) Види цін
    13) Підрозділи
    14) Контрагенти
    15) Товари контрагентів
    16) Регістр цін
    17) Регістр продаж
    18) Налаштування
    19) Системні помилки
    20) Системні сповіщення
    21) Користувачі
    22) Склади
    23) Робочі місця
    24) Методи оплати
    25) Екваєрингові термінали
    26) Фіскальні реєстратори
    27) Ваги
    28) Серії товарів
    29) Касові зміни
    30) Касові транзакції
    31) Регістр руху коштів
    32) Регістр продаж
    33) Документ прихідного ордеру (шапка, товари)
    34) Документ повернення товарів (шапка, товари)
    35) Документ ціноутворення (шапка, товари)
    36) Документ підбору товарів у чек (шапка, товари)
    37) Документ касового чеку (шапка, товари, платежі)
    38) Документ звіт касової зміни (шапка, товари)
    39) Документ інвентаризації (шапка, товари)
    40) Документ списання (шапка, товари)
*/

export default async function SaveReplicationResponse(data) {
    const transaction = await Database.instance.transaction({ isolationLevel: Transaction.ISOLATION_LEVELS.SERIALIZABLE })
    try {
        const {
            roles, taxes, transactions_articles,
            products_categories, products_kinds, products_packings, products_characteristic, products_producers, products_active_substances, products, barcodes,
            price_types, subdivisions, counterparties, counterparties_products,
            registers_product_price, registers_product_stock,
            settings, app_errors, app_notifications,
            staff, warehouses, workplaces, payment_methods,
            acquiring_terminals, fiscal_registers, scales,
            series, shifts, transactions,
            registers_money,
            registers_sales,
            documents_arrival, documents_arrival_goods,
            documents_return, documents_return_goods,
            documents_pricing, documents_pricing_goods,
            documents_draft, documents_draft_goods,
            documents_receipt, documents_receipt_goods, documents_receipt_payments,
            documents_report, documents_report_goods,
            documents_inventory, documents_inventory_goods,
            documents_writeoff, documents_writeoff_goods,
        } = data

        // Збереження даних не залежних від зовнішніх ключів
        for (let i = 0; subdivisions && i < subdivisions.length; i++) {
            await Database.Subdivisions.upsert(subdivisions[i], { transaction, silent: true })
        }
        for (let i = 0; settings && i < settings.length; i++) {
            await Database.Settings.upsert(settings[i], { transaction, silent: true })
        }
        for (let i = 0; fiscal_registers && i < fiscal_registers.length; i++) {
            await Database.FiscalRegisters.upsert(fiscal_registers[i], { transaction, silent: true })
        }
        for (let i = 0; acquiring_terminals && i < acquiring_terminals.length; i++) {
            await Database.AcquiringTerminals.upsert(acquiring_terminals[i], { transaction, silent: true })
        }
        for (let i = 0; scales && i < scales.length; i++) {
            await Database.Scales.upsert(scales[i], { transaction, silent: true })
        }
        for (let i = 0; counterparties && i < counterparties.length; i++) {
            await Database.Counterparties.upsert(counterparties[i], { transaction, silent: true })
        }
        for (let i = 0; price_types && i < price_types.length; i++) {
            await Database.PriceTypes.upsert(price_types[i], { transaction, silent: true })
        }
        for (let i = 0; roles && i < roles.length; i++) {
            await Database.Roles.upsert(roles[i], { transaction, silent: true })
        }
        for (let i = 0; taxes && i < taxes.length; i++) {
            await Database.Taxes.upsert(taxes[i], { transaction, silent: true })
        }
        for (let i = 0; transactions_articles && i < transactions_articles.length; i++) {
            await Database.TransactionsArticles.upsert(transactions_articles[i], { transaction, silent: true })
        }
        for (let i = 0; products_categories && i < products_categories.length; i++) { // ProductCategories
            await Database.ProductCategories.upsert(products_categories[i], { transaction, silent: true })
        }
        for (let i = 0; products_kinds && i < products_kinds.length; i++) {
            await Database.ProductKinds.upsert(products_kinds[i], { transaction, silent: true })
        }
        for (let i = 0; products_characteristic && i < products_characteristic.length; i++) {
            await Database.ProductCharacteristic.upsert(products_characteristic[i], { transaction, silent: true })
        }
        for (let i = 0; products_producers && i < products_producers.length; i++) {
            await Database.ProductProducers.upsert(products_producers[i], { transaction, silent: true })
        }
        for (let i = 0; products_active_substances && i < products_active_substances.length; i++) {
            await Database.ProductActiveSubstances.upsert(products_active_substances[i], { transaction, silent: true })
        }
        for (let i = 0; app_errors && i < app_errors.length; i++) {
            await Database.System.Errors.upsert(app_errors[i], { transaction, silent: true })
        }
        for (let i = 0; app_notifications && i < app_notifications.length; i++) {
            await Database.System.Notifications.upsert(app_notifications[i], { transaction, silent: true })
        }

        // Збереження даних залежних від зовнішніх ключів у порядку залежності
        for (let i = 0; staff && i < staff.length; i++) { // Roles
            await Database.Staff.upsert(staff[i], { transaction, silent: true })
        }
        for (let i = 0; warehouses && i < warehouses.length; i++) { // PriceTypes
            await Database.Warehouses.upsert(warehouses[i], { transaction, silent: true })
        }
        for (let i = 0; payment_methods && i < payment_methods.length; i++) { // AcquiringTerminals
            await Database.PaymentMethods.upsert(payment_methods[i], { transaction, silent: true })
        }
        for (let i = 0; workplaces && i < workplaces.length; i++) { // FiscalRegisters & Scales & PriceTypes
            await Database.Workplaces.upsert(workplaces[i], { transaction, silent: true })
        }
        for (let i = 0; products && i < products.length; i++) { // Taxes & ProductCategories & ProductKinds & ProductCharacteristic & ProductProducers & ProductActiveSubstances
            await Database.Products.upsert(products[i], { transaction, silent: true })
        }
        for (let i = 0; products_packings && i < products_packings.length; i++) { // Products
            await Database.ProductPacking.upsert(products_packings[i], { transaction, silent: true })
        }
        for (let i = 0; series && i < series.length; i++) { // Products
            await Database.Series.upsert(series[i], { transaction, silent: true })
        }
        for (let i = 0; barcodes && i < barcodes.length; i++) { // Products & ProductPacking
            await Database.Barcodes.upsert(barcodes[i], { transaction, silent: true })
        }
        for (let i = 0; counterparties_products && i < counterparties_products.length; i++) {
            await Database.CounterpartiesProducts.upsert(counterparties_products[i], { transaction, silent: true })
        }

        for (let i = 0; shifts && i < shifts.length; i++) { // Workplaces & Staff & FiscalRegisters
            await Database.Documents.Shifts.upsert(shifts[i], { transaction, silent: true })
        }
        for (let i = 0; transactions && i < transactions.length; i++) { // Shifts && TransactionsArticles
            await Database.Documents.ShiftsTransactions.upsert(transactions[i], { transaction, silent: true })
        }


        for (let i = 0; documents_arrival && i < documents_arrival.length; i++) { // Warehouses & PriceTypes & Counterparties
            await Database.Documents.Arrival.upsert(documents_arrival[i], { transaction, silent: true })
        }
        for (let i = 0; documents_arrival_goods && i < documents_arrival_goods.length; i++) { // Documents.Arrival && Products
            await Database.Documents.ArrivalGoods.upsert(documents_arrival_goods[i], { transaction, silent: true })
        }

        for (let i = 0; documents_return && i < documents_return.length; i++) {// Warehouses & PriceTypes & Counterparties
            await Database.Documents.Return.upsert(documents_return[i], { transaction, silent: true })
        }
        for (let i = 0; documents_return_goods && i < documents_return_goods.length; i++) {// Documents.Return && Products
            await Database.Documents.ReturnGoods.upsert(documents_return_goods[i], { transaction, silent: true })
        }

        for (let i = 0; documents_pricing && i < documents_pricing.length; i++) {// Documents.Arrival && PriceTypes
            await Database.Documents.Pricing.upsert(documents_pricing[i], { transaction, silent: true })
        }
        for (let i = 0; documents_pricing_goods && i < documents_pricing_goods.length; i++) {// Documents.Pricing && Products && ProductsPackings
            await Database.Documents.PricingGoods.upsert(documents_pricing_goods[i], { transaction, silent: true })
        }

        for (let i = 0; documents_draft && i < documents_draft.length; i++) { // Workplaces & Staff
            await Database.Documents.Draft.upsert(documents_draft[i], { transaction, silent: true })
        }
        for (let i = 0; documents_draft_goods && i < documents_draft_goods.length; i++) { // Documents.Draft & Products
            await Database.Documents.DraftGoods.upsert(documents_draft_goods[i], { transaction, silent: true })
        }

        for (let i = 0; documents_receipt && i < documents_receipt.length; i++) { // Workplaces & Staff & Shift & FiscalRegisters
            await Database.Documents.Receipt.upsert(documents_receipt[i], { transaction, silent: true })
        }
        for (let i = 0; documents_receipt_goods && i < documents_receipt_goods.length; i++) { // Documents.Receipt & Products
            await Database.Documents.ReceiptGoods.upsert(documents_receipt_goods[i], { transaction, silent: true })
        }
        for (let i = 0; documents_receipt_payments && i < documents_receipt_payments.length; i++) { // Documents.Receipt & PaymentMethods
            await Database.Documents.ReceiptPayment.upsert(documents_receipt_payments[i], { transaction, silent: true })
        }
        for (let i = 0; documents_report && i < documents_report.length; i++) { // Workplaces & Staff & Shift & Warehouses
            await Database.Documents.Report.upsert(documents_report[i], { transaction, silent: true })
        }
        for (let i = 0; documents_report_goods && i < documents_report_goods.length; i++) { // Documents.Report & PaymentMethods
            await Database.Documents.ReportGoods.upsert(documents_report_goods[i], { transaction, silent: true })
        }
        for (let i = 0; documents_inventory && i < documents_inventory.length; i++) { // Subdivision & Warehouse
            await Database.Documents.Inventory.upsert(documents_inventory[i], { transaction, silent: true })
        }
        for (let i = 0; documents_inventory_goods && i < documents_inventory_goods.length; i++) { // Documents.Inventory & Product
            await Database.Documents.InventoryGoods.upsert(documents_inventory_goods[i], { transaction, silent: true })
        }
        for (let i = 0; documents_writeoff && i < documents_writeoff.length; i++) { // Subdivision & Warehouse
            await Database.Documents.Writeoff.upsert(documents_writeoff[i], { transaction, silent: true })
        }
        for (let i = 0; documents_writeoff_goods && i < documents_writeoff_goods.length; i++) { // Documents.Writeoff & Product
            await Database.Documents.WriteoffGoods.upsert(documents_writeoff_goods[i], { transaction, silent: true })
        }

        for (let i = 0; registers_product_stock && i < registers_product_stock.length; i++) { // Products
            await Database.Registers.ProductStock.upsert(registers_product_stock[i], { transaction, silent: true })
        }
        for (let i = 0; registers_product_price && i < registers_product_price.length; i++) {// Products
            await Database.Registers.ProductPrice.upsert(registers_product_price[i], { transaction, silent: true })
        }
        if (registers_product_price?.length) {
            const distinct = []
            registers_product_price.filter(({ product_id, series_id, packing_id, price_type_id }) => {
                const exists = distinct.find(it => it.id === `${product_id + series_id + packing_id + price_type_id}`)
                if (!exists) distinct.push({ id: `${product_id + series_id + packing_id + price_type_id}`, product_id, series_id, packing_id, price_type_id })
            })
            for (let j = 0; distinct && j < distinct.length; j++) {// Products
                const { id, ...where } = distinct[j]
                await ProductPriceRegisterValuesHelper.recalculate({ where, transaction })
            }
        }
        for (let i = 0; registers_money && i < registers_money.length; i++) { // Subdivisions & Workplaces & TransactionsArticles
            await Database.Registers.Money.upsert(registers_money[i], { transaction, silent: true })
        }
        for (let i = 0; registers_sales && i < registers_sales.length; i++) { // Subdivisions & Workplaces & Staff & Products & ProductsCategories
            await Database.Registers.Sales.upsert(registers_sales[i], { transaction, silent: true })
        }
        await transaction.commit()
    } catch (error) {
        await transaction.rollback()
        throw error
    }
}